
;**************************************************************************
;*                    Temperature Logger                                  *
;**************************************************************************
;**************************************************************************
;*          v 1.56 - 29.08.2014                                           *
;*                                                                        *
;*          LCD & PS/2 keyboard routines included                         *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC18LF25K22 used                                     *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************



;------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------
;change history
;v1.17
;		clean-up
;		problem with sensor D fixed (it was only a display problem)
;v1.21		basic memory access opeations implemented
;v1.26		creation of temperature record (01) included
;v1.30		clean-up
;v1.41		Flash memory 1MB supported
;v1.42		Identical copy as v1.41
;v1.43		USB HEX File output included
;v1.44		optimizing of check_files routine
;v1.45		"Memory" menu added
;		procesor changed to PIC18LF25K22 (memory)
;v1.52		functionality almost finalized
;v1.53		Firmware clean-up
;		few minor bugs corrected
;v1.54		measurement interval 1hrs included
;v1.55		Sensor deregistration functionality included
;------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------



;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)

;keyboard
;	kbdclk  -- RB7
;	kbddata -- RB6


 INCLUDE "P18LF25K22.INC"

 CONFIG		FOSC	= INTIO67
 CONFIG		BORV	= 190
 CONFIG		BOREN	= OFF
 CONFIG		PWRTEN	= ON
 CONFIG		WDTEN	= OFF
 CONFIG		PBADEN	= OFF
 CONFIG		MCLRE	= INTMCLR
 CONFIG		STVREN	= OFF
 CONFIG		XINST	= ON
 CONFIG		DEBUG	= OFF
 CONFIG		CP0	= OFF
 CONFIG		CP1	= OFF
 CONFIG		CPB	= OFF
 CONFIG		CPD	= OFF
 CONFIG		WRT0	= OFF
 CONFIG		WRT1	= OFF
 CONFIG		WRTB	= OFF
 CONFIG		WRTC	= OFF
 CONFIG		WRTD	= OFF
 CONFIG		EBTR0	= OFF
 CONFIG		EBTR1	= OFF
 CONFIG		EBTRB	= OFF


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'00'		;delay routine
TIMER2		EQU	H'01'		; "	"	"

v_ttt		EQU	H'02'
c_tmp		EQU	H'03'
data_tmp	EQU	H'04'
disp_pos	EQU	H'05'

kbd_buffer	EQU	H'06'
kbd_out		EQU	H'07'
kbd_parity	EQU	H'08'
kbd_bb		EQU	H'09'		;keyboard bit buffer


v_comm		EQU	H'0A'
v_b_in		EQU	H'0B'

tmp_ot		EQU	H'0C'
v_cursor	EQU	H'0D'
b_ax_h		EQU	H'0E'
b_ax		EQU	H'0F'

v_tst1		EQU	H'10'
v_tst2		EQU	H'11'
v_tst3		EQU	H'12'

v_i2c_add	EQU	H'13'

v_temp1		EQU	H'14'
v_temp2		EQU	H'15'


ow_buffer	EQU	H'19'
ow_ROM_0	EQU	H'1A'
ow_ROM_1	EQU	H'1B'
ow_ROM_2	EQU	H'1C'
ow_ROM_3	EQU	H'1D'
ow_ROM_4	EQU	H'1E'
ow_ROM_5	EQU	H'1F'
ow_ROM_6	EQU	H'20'
ow_ROM_7	EQU	H'21'

ow_m1		EQU	H'22'
ow_m2		EQU	H'23'
ow_m3		EQU	H'24'
ow_m4		EQU	H'25'
ow_m5		EQU	H'26'
ow_m6		EQU	H'27'
ow_m7		EQU	H'28'
ow_m8		EQU	H'29'
ow_m9		EQU	H'2A'

ds2484_dc	EQU	H'2B'
ds2484_pc	EQU	H'2C'
ds2484_st	EQU	H'2D'
ds2484_data	EQU	H'2E'

;---------------------------------------------------------------------

v_re_run	EQU	H'2F'			;re-run

v_spi_buffi	EQU	H'30'
v_spi_conf	EQU	H'31'


v_md_status2	EQU	H'32'
				;bit0 = 1 -> Memory full

				;bit6 & 7 -> type of USB transfer

;---------------------------------------------------
i2ce_cursor	EQU	H'33'
i2ce_op		EQU	H'34'
;---------------------------------------------------
;---------------------------------------------------
;---------------------------------------------------
v_i2c_devs	EQU	H'35'
v_i2c_actd	EQU	H'36'


;redefining 35h & 36h

v_dic		EQU	H'35'
v_usb_sw	EQU	H'36'

;---------------------------------------------------
;---------------------------------------------------

v_sensors	EQU	H'37'
				;bit0 = 1 -> Sensor A is on the bus
				;bit1 = 1 -> Sensor B is on the bus
				;bit2 = 1 -> Sensor C is on the bus
				;bit3 = 1 -> Sensor D is on the bus
				;bit4 = 1 -> Sensor A is registered
				;bit5 = 1 -> Sensor B is registered
				;bit6 = 1 -> Sensor C is registered
				;bit7 = 1 -> Sensor D is registered


ee_add		EQU	H'38'
ee_buffer	EQU	H'39'

;----------------------------------------------------------------------------------
v_md_status	EQU	H'3A'	;Logger - Status information
				;bit 0    - Keyboard present flag
				;bit 1    - mesurement in progress flag
				;bit 2, 3, 4 - Memory type:
					;bit4,3,2 = 000 -- no memory
					;bit4,3,2 = 001 -- 64 kB NVRAM
					;bit4,3,2 = 010 -- 64 kB EEPROM
					;bit4,3,2 = 100 --  8 MB Flash
;----------------------------------------------------------------------------------


;---------------------------------------------------
;---------------------------------------------------
;temperature from sensors A-B
;temperature format:
;   bit 7     = sign (0 = plus / 1 = minus)
;   bit 6 - 0 = temperature value
;   example:
;       B'00010000' = +16C
;       B'00010001' = +17C
;       B'10010000' = -16C
;       B'10010001' = -17C
;---------------------------------------------------
v_i1		EQU	H'3B'
v_i2		EQU	H'3C'
v_i3		EQU	H'3D'
v_i4		EQU	H'3E'
;---------------------------------------------------

v_m_add0	EQU	H'3F'
v_m_add1	EQU	H'40'
v_m_add2	EQU	H'41'

;---------------------------------------------------
;---------------------------------------------------
;used for 1-wire Search ROM algorithm
;---------------------------------------------------
v_ow_dc		EQU	H'42'		;number of 1-wire devices found
v_ow_act_dir	EQU	H'43'

v_owc0		EQU	H'44'
v_owc1		EQU	H'45'
v_owc2		EQU	H'46'
v_owc3		EQU	H'47'
v_owc4		EQU	H'48'
v_owc5		EQU	H'49'
v_owc6		EQU	H'4A'
v_owc7		EQU	H'4B'
v_owcc		EQU	H'4C'

v_ow_curr	EQU	H'4D'


v_owd0		EQU	H'50'
v_owd1		EQU	H'51'
v_owd2		EQU	H'52'
v_owd3		EQU	H'53'
v_owd4		EQU	H'54'
v_owd5		EQU	H'55'
v_owd6		EQU	H'56'
v_owd7		EQU	H'57'

v_owdc		EQU	H'58'		;direction for search - current byte
v_rom_id_c	EQU	H'59'		;current byte of the ROM ID
;---------------------------------------------------
;---------------------------------------------------
;RTC variables --------------------
v_rtc_d1	EQU	H'42'
v_rtc_d2	EQU	H'43'
v_rtc_m1	EQU	H'44'
v_rtc_m2	EQU	H'45'
v_rtc_y1	EQU	H'46'
v_rtc_y2	EQU	H'47'
v_rtc_y3	EQU	H'48'
v_rtc_y4	EQU	H'49'
v_rtc_h1	EQU	H'4A'
v_rtc_h2	EQU	H'4B'
v_rtc_n1	EQU	H'4C'
v_rtc_n2	EQU	H'4D'
v_rtc_s1	EQU	H'4E'
v_rtc_s2	EQU	H'4F'

v_rtc_sec	EQU	H'50'
v_rtc_min	EQU	H'51'
v_rtc_hrs	EQU	H'52'
v_rtc_wday	EQU	H'53'		;day in the week
v_rtc_day	EQU	H'54'
v_rtc_mnt	EQU	H'55'
v_rtc_year	EQU	H'56'

v_rtc_wd	EQU	H'57'
;---------------------------------------------------
;---------------------------------------------------
v_tmp1		EQU	H'58'		;temp variable - general usage
v_tmp2		EQU	H'59'		;temp variable - special usage - no RTC possible

v_rtc_t1	EQU	H'59'
v_rtc_t2	EQU	H'5A'
;---------------------------------------------------
v_file		EQU	H'5B'		;current file ID
v_num_files	EQU	H'5C'		;total number of files
v_interval	EQU	H'5E'

v_rec_lsb	EQU	H'16'
v_rec_msb	EQU	H'17'
v_temp2s	EQU	H'18'
;---------------------------------------------------


;last used RAM position: H'5A' / max: H'5F'


;100h - 120h -- OneWire ROM IDs
;200h - 210h -- Memory Record Buffer


;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.


;==========================================================================
; Text definitions
;==========================================================================
		ORG	100


t_release1	da	"Temperature logger      ", H'FE'
t_release2	da	"Firmware 1.56-29.08.2014", H'FE'

t_chkf		da	"Checking Files/Memory...", H'FE'

t_owtm1		da	"Too much devices found  ", H'FE'
t_owtm2		da	"Devices allowed: 0 - 4  ", H'FE'
t_usd1		da	"Not supported device ", H'FE'
t_usd2		da	"Device Family:", H'FE'

t_memf1		da	"Mem. Full ", H'FE'
t_memf1b	da	"Memory Full ", H'FE'

t_new_s1	da	"New sensor found", H'FE'
t_new_s2	da	"Register as: A-B-C-D ?", H'FE'
t_new_s3	da	"No registration possible ", H'FE'

t_sens_inf1	da	"Registered sensors: ", H'FE'
t_sens_inf2	da	"Sensors on the bus: ", H'FE'

t_lgmm1		da	"(S)ystem", H'FE'
t_lgmm2		da	"(F)ile", H'FE'

t_lgsm1		da	"(M)emory / (S)ensors", H'FE'
t_lgsm2		da	"(C)lock  / (R)eset  ", H'FE'

t_file1		da	"(N)ew/(L)ist", H'FE'
t_file2		da	"(R)ead all/(U)SB", H'FE'

t_fem2		da	"(R)ead / (U)SB", H'FE'

t_sm1		da	"Device (i)nformation", H'FE'
t_sm2		da	"(D)eregistration", H'FE'

t_sm5		da	"(D)eregister? ", H'FE'

t_usbm1		da	"Format(1) - Full  ", H'FE'
t_usbm2		da	"Format(2) - Slim / (H)EX", H'FE'

t_nsf1		da	"Unsupported flash found ", H'FE'
t_ssf1		da	"SST25VF080B found - 1MB ", H'FE'
t_nmm1		da	"No memory found ", H'FE'
t_nvram1	da	"NVRAM identified - 64kB ",H'FE'
t_eeprom1	da	"EEPROM identified  64kB ",H'FE'

t_fip1		da	"Current File ID: ", H'FE'
t_fip2		da	"(S)top? ", H'FE'

t_fi_rFF	da	"Empty record", H'FE'
t_fi_rXX	da	"Unknown record", H'FE'

t_mem2		da	"(I)nfo / (E)rase", H'FE'
t_erase1	da	"ENTER = Erase all Data", H'FE'
t_erase2	da	"ESC   = Back", H'FE'

t_spi		da	"(E)EPROM / (S)PI Editor ", H'FE'

t_kbd_p		da	"Keyboard connected", H'FE'
t_kbd_n		da	"Keyboard not found", H'FE'

t_cc1		da	"Memory Control Record ", H'FE'
t_cc2		da	"Control Record created", H'FE'

t_new_file1	da	"File ID : ", H'FE'
t_new_file2	da	"(P)eriod:     / (S)tart ", H'FE'

t_usb_h1	da	"Temperature logger file ", H'FE'

t_usb_01	da	"Time: ", H'FE'
t_usb_03	da	"Start of measurement: ", H'FE'
t_usb_04	da	"End of measurement: ", H'FE'
t_usb_oth	da	"Other Record", H'FE'
t_usb_minus	da	"------------------------------------------------------------------------", H'FE'
t_usb_irr	da	"Transfer interrupted... ", H'FE'
t_usbti1	da	"USB Transfer in progress", H'FE'

t_ds1820	da	"ROM code(CRC*SN--Fam):", H'FE'


t_i2c_cee1	da	"I2C critical error", H'FE'
t_i2c_ee1	da	"I2C communication error ", H'FE'


t_ee_save	da	"Saving data...", H'FE'
t_ee_addr	da	"A:1010000  Add:", H'FE'

t_i2ce1		da	"I2CE", H'FE'

t_spi_par1	da	"(I)dle Clock State: ", H'FE'

t_fse1		da	"Wrong File Structure", H'FE'
t_fse2		da	"System halt ", H'FE'

;==========================================================================
;==========================================================================

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		ORG 700

init_main	nop
		movlb	H'0F'

		lfsr	FSR2, H'000'		;FSR2 to start of bank 0

;oscilator setup
		movlw	B'01100000'		;Frequency = 8MHz  ( OSCON<7:5> = 110 )
		movwf	OSCCON

		clrf	INTCON			;disable interrupts

;PORTA set-up
		clrf	PORTA
		clrf	LATA
		clrf	ANSELA

		clrf	PORTB
		clrf	LATB
		clrf	ANSELB
		movlw	B'00000000'
		movwf	TRISA
		movlw	B'11111111'
		movwf	TRISB

		movlw	B'11000000'
		movwf	WPUB

;set-up for I2C Master operation for MSSP1
		clrf	ANSELC
		movlw	B'11111011'
		movwf	TRISC
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		clrf	v_usb_sw

		clrf	v_md_status
		clrf	v_md_status2
		call	i2c_init
		call	spi_init
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		call	disp_init
		call	w_release
		call	drr2
		call	drr2

		call	disp_clr1
		call	disp_clr2

;keaboard test

		movlw	H'ED'
		movwf	kbd_out
		call	send_ck
		call	dr01s			;ignore the ack from kbd and wait a little bit
		movlw	B'00000010'
		movwf	kbd_out
		call	send_ck

		btfsc	v_md_status,D'000'
		call	w_kbd_found		;no keyboard connected

		btfss	v_md_status,D'000'
		call	w_kbd_nfound		;keyboard present

		call	drr2

		movlw	H'FF'
		movwf	v_i1
		movwf	v_i2
		movwf	v_i3
		movwf	v_i4

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
; Start-up sequence:
;
;     1. Chceck the sensors
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
start_up1	call	disp_clr1
		call	disp_clr2

		clrf	v_re_run
		clrf	v_sensors

		call	check_regs

		call	ow_rs_execute

		movf	v_ow_dc,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	sc_fin			;there is no device on the bus

		clrf	v_ow_curr


sc_next1	lfsr	FSR0, H'100'

		movf	v_ow_curr,0
		xorwf	v_ow_dc,0
		btfsc	STATUS,Z
		goto	sc_fin			;we compared all sensors found

		incf	v_ow_curr,1

		movf	v_ow_curr,0
		movwf	c_tmp

sc_loop2	call	load_ROMID
		decfsz	c_tmp,1
		goto	sc_loop2

		call	match_sensor
		btfss	v_re_run,D'000'
		goto	sc_next1
		goto	start_up1		;there was a sensor registered - just start over

;------------------------------------------------------------------------------
sc_fin		nop
		call	w_sens_info

		call	drr2
		call	drr2


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
; Start-up sequence:
;
;     1. Check the Sensors - done
;     2. Check the Memory
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		call	disp_clr1
		call	disp_clr2


;1. execute Read ID instruction
		call	spi_cs0_en
		movlw	H'90'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send

		call	spi_send
		movf	v_spi_buffi,0
		xorlw	H'FF'
		btfss	STATUS,Z
		goto	sus_mem_flash		;we found a flash memory
;we got no response

		call	spi_cs0_dis

;2. find out, if there is something else
;2.1. execute WRDI (Write Disable for EEPROM / ignored by NVRAM) -- this lead to clearing the bit 0 in STATUS register
		call	spi_cs0_en
		movlw	B'00000100'
		call	spi_send
		call	spi_cs0_dis

;2.2. read Status register
		call	spi_cs0_en
		movlw	B'00000101'
		call	spi_send
		call	spi_send
		call	spi_cs0_dis
		movf	v_spi_buffi,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	no_memory
;-------------------------------------------------------------------------------
;we found something, assuming it is either 25AA512 (EEPROM 64kB) or 23LCV512 (NVRAM 64kB)
;3.1. Execute WREN
		call	ee_wren
;3.2 read Status register
		call	spi_cs0_en
		movlw	B'00000101'
		call	spi_send
		call	spi_send
		call	spi_cs0_dis
		btfss	v_spi_buffi,D'001'
		goto	m_NVRAM
		goto	m_EEPROM

;-------------------------------------------------------------------------------
;Execute Write Enable Instruction
;
ee_wren		call	spi_cs0_en
		movlw	B'00000110'			;WREN
		call	spi_send
		call	spi_cs0_dis
		return
;-------------------------------------------------------------------------------
;we was able to identify the device 23LCV512 (or similar)

m_NVRAM		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_nvram1
		movwf	TBLPTRH
		movlw	low t_nvram1
		movwf	TBLPTRL
		call	w_send_txt

;bit4,3,2 = 000 -- no memory
;bit4,3,2 = 001 -- 64 kB NVRAM
;bit4,3,2 = 010 -- 64 kB EEPROM
;bit4,3,2 = 100 --  8 MB Flash
		bsf	v_md_status,D'002'		;NVRAM
		bcf	v_md_status,D'003'
		bcf	v_md_status,D'004'

		call	drr2
		call	drr2
		goto	tl_ms01_p

;-------------------------------------------------------------------------------
;we was able to identify the device 25AA512 (or similar)

m_EEPROM	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_eeprom1
		movwf	TBLPTRH
		movlw	low t_eeprom1
		movwf	TBLPTRL
		call	w_send_txt

;bit4,3,2 = 000 -- no memory
;bit4,3,2 = 001 -- 64 kB NVRAM
;bit4,3,2 = 010 -- 64 kB EEPROM
;bit4,3,2 = 100 --  8 MB Flash
		bcf	v_md_status,D'002'
		bsf	v_md_status,D'003'		;EEPROM
		bcf	v_md_status,D'004'

		call	drr2
		call	drr2
		goto	tl_ms01_p

;-------------------------------------------------------------------------------
sus_mem_flash	movf	v_spi_buffi,0
		movwf	v_ttt			;first ID
		call	spi_send		;get second ID
		call	spi_cs0_dis
;
		movf	v_ttt,0
		xorlw	H'BF'
		btfss	STATUS,Z
		goto	sus_mem_nots		;we found not supported flash memory
		movf	v_spi_buffi,0
		xorlw	H'8E'
		btfss	STATUS,Z
		goto	sus_mem_nots		;we found not supported flash memory
;-------------------------------------------------------------------------------
;we was able to identify the device SST25VF080B

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_ssf1
		movwf	TBLPTRH
		movlw	low t_ssf1
		movwf	TBLPTRL
		call	w_send_txt

;bit4,3,2 = 000 -- no memory
;bit4,3,2 = 001 -- 64 kB NVRAM
;bit4,3,2 = 010 -- 64 kB EEPROM
;bit4,3,2 = 100 --  8 MB Flash
		bcf	v_md_status,D'002'
		bcf	v_md_status,D'003'
		bsf	v_md_status,D'004'		;Flash


;unlock Flash memory for write operations
		call	spi_cs0_en
		movlw	H'50'
		call	spi_send
		call	spi_cs0_dis

		call	spi_cs0_en
		movlw	H'01'
		call	spi_send
		movlw	H'00'
		call	spi_send
		call	spi_cs0_dis


		call	drr2
		call	drr2
		goto	tl_ms01_p
;-------------------------------------------------------------------------------
sus_mem_nots	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_nsf1
		movwf	TBLPTRH
		movlw	low t_nsf1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	wm_I
		call	wm_D
		call	wm_dvojbodka
		call	wm_space

		movf	v_ttt,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		movf	v_spi_buffi,0
		movwf	kbd_buffer
		call	write_hex

;bit4,3,2 = 000 -- no memory
;bit4,3,2 = 001 -- 64 kB NVRAM
;bit4,3,2 = 010 -- 64 kB EEPROM
;bit4,3,2 = 100 --  8 MB Flash
		bcf	v_md_status,D'002'
		bcf	v_md_status,D'003'
		bcf	v_md_status,D'004'

		call	drr2
		call	drr2
		call	drr2
		call	drr2
		goto	tl_ms01_p
;------------------------------------------------------------------------------

no_memory	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_nmm1
		movwf	TBLPTRH
		movlw	low t_nmm1
		movwf	TBLPTRL
		call	w_send_txt

;bit4,3,2 = 000 -- no memory
;bit4,3,2 = 001 -- 64 kB NVRAM
;bit4,3,2 = 010 -- 64 kB EEPROM
;bit4,3,2 = 100 --  8 MB Flash
		bcf	v_md_status,D'002'
		bcf	v_md_status,D'003'
		bcf	v_md_status,D'004'

		call	drr2
		call	drr2
		call	drr2
		call	drr2
		goto	tl_ms01_p


;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Memory access routines - "BIOS"
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
open_for_read	call	spi_cs0_dis
		nop

		call	spi_cs0_en
		movlw	H'03'			;SPI command READ
		call	spi_send
;send MSB for Flash only -----------------
		btfsc	v_md_status,D'004'
		movf	v_m_add2,0
		btfsc	v_md_status,D'004'
		call	spi_send
;-----------------------------------------

		movf	v_m_add1,0
		call	spi_send
		movf	v_m_add0,0
		call	spi_send

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_rec_next	movlw	D'016'
		movwf	c_tmp

		lfsr	FSR0, H'200'		;start of the record buffer

sx001		call	spi_send
		movf	v_spi_buffi,0
		movwf	POSTINC0
		decfsz	c_tmp,1
		goto	sx001

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_inc_add16	btfsc	v_md_status2,D'000'
		return					;do nothing, if memory full flag is set

		call	s_inc_add16i

		btfss	v_md_status,D'004'
		goto	si16n

;check of add2 for 1MB Flash
		movf	v_m_add2,0
		xorlw	H'0F'
		btfss	STATUS,Z
		return

si16n		movf	v_m_add1,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return

;last 16 sectors will not be used

		bsf	v_md_status2,D'000'		;memory full

		btfss	v_md_status,D'001'
		return

		call	file_close
		bcf	v_md_status,D'001'
		call	check_files

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_inc_add16i	movlw	H'10'
		addwf	v_m_add0,1
		btfss	STATUS,Z
		return
		incf	v_m_add1,1
		btfss	STATUS,Z
		return
		incf	v_m_add2,1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_inc_add1i	incf	v_m_add0,1
		btfss	STATUS,Z
		return
		incf	v_m_add1,1
		btfss	STATUS,Z
		return
		incf	v_m_add2,1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_dec_add1i	decf	v_m_add0,1
		movf	v_m_add0,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return
		decf	v_m_add1,1
		movf	v_m_add1,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return
		decf	v_m_add2,1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;read record from memory to the buffer
;   record address (address of first byte) is stored in:
;      v_m_add0	- LSB
;      v_m_add1
;      v_m_add2 - MSB
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_rec_read	call	open_for_read
		call	s_rec_next
		call	spi_cs0_dis
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
init_nvram	call	spi_cs0_en
		movlw	H'02'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send

		movlw	B'11000000'
		call	send_comm

		movlw	H'FF'
		movwf	v_temp1

invrl2		movf	v_temp1,0
		andlw	H'0F'
		btfss	STATUS,Z
		goto	invrl3
		movlw	H'FF'
		call	send_data

invrl3		movlw	H'FF'
		movwf	v_temp2

invrl1		movlw	H'FF'
		call	spi_send
		decfsz	v_temp2,1
		goto	invrl1
		decfsz	v_temp1,1
		goto	invrl2

		call	spi_cs0_dis
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;write record from buffer to the memory
;   record address (address of first byte) is stored in:
;      v_m_add0	- LSB
;      v_m_add1
;      v_m_add2 - MSB
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_rec_write	btfsc	v_md_status,D'004'
		goto	s_rec_write_f		;for Flash we have to follow different logic

		btfss	v_md_status,D'002'
		call	ee_wren			;execute WREN for EEPROM

		call	spi_cs0_en
		movlw	H'02'			;SPI command WRITE
		call	spi_send

		movf	v_m_add1,0
		call	spi_send
		movf	v_m_add0,0
		call	spi_send

		movlw	D'016'
		movwf	c_tmp

		lfsr	FSR0, H'200'		;start of the record buffer

sx002		movf	POSTINC0,0
		call	spi_send
		decfsz	c_tmp,1
		goto	sx002
		call	spi_cs0_dis

		btfss	v_md_status,D'002'
		call	dr2			;wait a little bit for EEPROM

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;write 16 bytes - Flash variant
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_rec_write_f	nop

		movlw	D'016'
		movwf	c_tmp

		lfsr	FSR0, H'200'		;start of the record buffer

sxf002		call	s_flash_wbyte
		incf	v_m_add0,1
		decfsz	c_tmp,1
		goto	sxf002

		movlw	H'10'
		subwf	v_m_add0,1

		return
;------------------------------------------------------------------------------
s_flash_wbyte	call	ee_wren			;execute WREN

		call	spi_cs0_en

		movlw	H'02'			;SPI command WRITE
		call	spi_send

		movf	v_m_add2,0
		call	spi_send
		movf	v_m_add1,0
		call	spi_send
		movf	v_m_add0,0
		call	spi_send

		movf	POSTINC0,0
		call	spi_send

		call	spi_cs0_dis

		call	d1_f

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_load_time	lfsr	FSR0,H'20A'
		movf	POSTINC0,0
		movwf	v_rtc_year
		movf	POSTINC0,0
		movwf	v_rtc_mnt
		movf	POSTINC0,0
		movwf	v_rtc_day
		movf	POSTINC0,0
		movwf	v_rtc_hrs
		movf	POSTINC0,0
		movwf	v_rtc_min
		movf	POSTINC0,0
		movwf	v_rtc_sec
		return
;------------------------------------------------------------------------------
s_fill_time2	movlw	B'11010000'		;RTC = 1101000x
		movwf	v_i2c_add
		call	rtc_i2c_read

s_fill_time	lfsr	FSR0,H'20A'
		movf	v_rtc_year,0
		movwf	POSTINC0		;A
		movf	v_rtc_mnt,0
		movwf	POSTINC0		;B
		movf	v_rtc_day,0
		movwf	POSTINC0		;C
		movf	v_rtc_hrs,0
		movwf	POSTINC0		;D
		movf	v_rtc_min,0
		movwf	POSTINC0		;E
		movf	v_rtc_sec,0
		movwf	POSTINC0		;F
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_control_rec	movf	v_md_status,0
		andlw	B'00011100'
		btfsc	STATUS,Z
		return

		clrf	v_m_add0
		clrf	v_m_add1
		clrf	v_m_add2

		call	s_rec_read

		lfsr	FSR0,H'200'
		movf	POSTINC0,0
		xorlw	H'AA'			;first byte of the control record
		btfss	STATUS,Z
		goto	s_create_cc

		movf	POSTINC0,0
		xorlw	H'55'			;first byte of the control record
		btfss	STATUS,Z
		goto	s_create_cc

		call	s_load_time
		call	rtc_translate

		call	w_cc_info
		call	drr2
		call	drr2

		return
;------------------------------------------------------------------------------
s_create_cc	btfsc	v_md_status,D'002'
		call	init_nvram

		lfsr	FSR0,H'200'

;fill the record buffer
		movlw	H'AA'
		movwf	POSTINC0
		movlw	H'55'
		movwf	POSTINC0
		movlw	H'FF'
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0
		movwf	POSTINC0

		call	s_fill_time2

		clrf	v_m_add0
		clrf	v_m_add1
		clrf	v_m_add2

		call	s_rec_write

		call	rtc_translate
		call	w_cc_info2
		call	drr2
		call	drr2

		return

;------------------------------------------------------------------------------
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;------------------------------------------------------------------------------
w_cc_info	call	disp_clr1
		call	disp_clr2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_cc1
		movwf	TBLPTRH
		movlw	low t_cc1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	w_rtcm_dt

		return

;------------------------------------------------------------------------------
w_cc_info2	call	disp_clr1
		call	disp_clr2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_cc2
		movwf	TBLPTRH
		movlw	low t_cc2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		call	w_rtcm_dt

		return

;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File access routines - "DOS"
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;------------------------------------------------------------------------------
check_files	call	disp_clr1
		call	disp_clr2
		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_chkf
		movwf	TBLPTRH
		movlw	low t_chkf
		movwf	TBLPTRL
		call	w_send_txt

		clrf	v_file
		clrf	v_num_files

		movlw	H'FF'
		movwf	v_i1
		movwf	v_i2
		movwf	v_i3
		movwf	v_i4

		movf	v_md_status,0
		andlw	B'00011100'
		btfsc	STATUS,Z
		return				;no memory in use

;find last file ID
		clrf	v_m_add2
		clrf	v_m_add1
		movlw	H'10'
		movwf	v_m_add0

		call	open_for_read

cf_loop1	call	s_rec_next

		lfsr	FSR0, H'200'		;start of the record buffer
		movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	end_of_umem
		movf	INDF0,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	start_of_file

		call	s_inc_add16
		call	inc_rec_no		;record number in variables l_rec_msb-v_rec_lsb is valid only for open file
		goto	cf_loop1

;------------------------------------------------------------------------------
;end of used memory - empty record
;here we have:
;	- bit 1 of v_md_status		have valid value
;	- v_num_values			number of files
;	- v_file 			last file ID in case file is in progress
;	- v_m_add2 - v_m_add0		start address of first free record

end_of_umem	call	spi_cs0_dis		;close the connection
		return

;------------------------------------------------------------------------------
;Start of File record found
start_of_file	lfsr	FSR0, H'201'
		bcf	v_md_status,D'001'

		movf	POSTINC0,0
		movwf	v_num_files		;current file ID

		movf	POSTINC0,0
		xorlw	H'FF'
		btfss	STATUS,Z
		goto	sof_finished
;--------------------------------------------------------------------
		bsf	v_md_status,D'001'	;file is not finished - we set the "mesurement in progress" flag
;--------------------------------------------------------------------
;in case this file is "in progress", we found some additional info

		clrf	v_rec_lsb
		clrf	v_rec_msb

		movf	v_num_files,0
		movwf	v_file			;current file ID
		lfsr	FSR0, H'206'
		movf	INDF0,0
		movwf	v_interval

		call	store_pointer

		call	s_inc_add16
		goto	cf_loop1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;file is closed already - now we can move the current read position according to file length
sof_finished	call	sof_finished_g
		goto	cf_loop1

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;1 - based on number of records calculate what we have to add to the current address (number of record * 16)
sof_finished_g	clrf	v_tst3
		lfsr	FSR0, H'203'
		movf	POSTINC0,0
		movwf	v_tst1			;File length - LSB
		movf	POSTINC0,0
		movwf	v_tst2			;File length - MSB

		rlcf	v_tst1,1
		rlcf	v_tst2,1
		rlcf	v_tst3,1

		rlcf	v_tst1,1
		rlcf	v_tst2,1
		rlcf	v_tst3,1

		rlcf	v_tst1,1
		rlcf	v_tst2,1
		rlcf	v_tst3,1

		rlcf	v_tst1,1
		rlcf	v_tst2,1
		rlcf	v_tst3,1

		bcf	v_tst1,D'000'
		bcf	v_tst1,D'001'
		bcf	v_tst1,D'002'
		bcf	v_tst1,D'003'

;2 - add the file length (in bytes) to the current address

		movf	v_tst1,0
		addwf	v_m_add0,1

		movf	v_tst2,0
		addwfc	v_m_add1,1

		movf	v_tst3,0
		addwfc	v_m_add2,1

;3 - re-open the read access at new computed position

		call	spi_cs0_dis

		call	open_for_read

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>>>>      File Open           <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;open a file with File ID (File ID is stored in v_file)


file_open	call	spi_cs0_dis

;address of the first file

		clrf	v_m_add2
		clrf	v_m_add1
		movlw	H'10'
		movwf	v_m_add0

		call	open_for_read

foo_loop2	call	s_rec_next


		lfsr	FSR0, H'200'		;start of the record buffer
		movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	file_not_found

		movf	POSTINC0,0
		xorlw	H'03'
		btfss	STATUS,Z
		goto	f_struct_err

		movf	POSTINC0,0
		xorwf	v_file,0
		btfss	STATUS,Z
		goto	foo_loop2prep
;--------------------------------------------------
;we found the file
;we got it - in buffer is the header record (type 03) of requested file

		return
;------------------------------------------------------------------------------
foo_loop2prep	movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	file_not_found		;this file is not closed -> means it is the last one --> we didn't found the requested one

		call	s_inc_add16i
		call	sof_finished_g

		goto	foo_loop2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;File not found
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
file_not_found	nop
		movlw	H'FF'
		movwf	v_file
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;File structure error - we expected record type 03 and found something else
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
f_struct_err	nop
		call	disp_clr1
		call	disp_clr2

		movlw	B'10000000'
		call	send_comm
		movlw	high t_fse1
		movwf	TBLPTRH
		movlw	low t_fse1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'
		call	send_comm
		movlw	high t_fse2
		movwf	TBLPTRH
		movlw	low t_fse2
		movwf	TBLPTRL
		call	w_send_txt

fin03		goto	fin03
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>>>>     Create a new File    <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
file_create	lfsr	FSR0, H'200'

		movlw	H'03'
		movwf	POSTINC0
		incf	v_num_files,1
		movf	v_num_files,0
		movwf	v_file
		movwf	POSTINC0
		movlw	H'FF'
		movwf	POSTINC0		;file in progress (byte 02)
		movwf	POSTINC0		;(03)
		movwf	POSTINC0		;(04)
		movwf	POSTINC0		;(05)
		movf	v_interval,0
		movwf	POSTINC0		;(06)
		movlw	H'FF'
		movwf	POSTINC0		;(07)
		movwf	POSTINC0		;(08)
		movwf	POSTINC0		;(09)

		call	s_fill_time2

		call	s_rec_write

		bsf	v_md_status,D'001'

		clrf	v_rec_lsb
		clrf	v_rec_msb

		call	store_pointer
		call	s_inc_add16		;set pointer to the next free record

		return


;-- store the pointer to the start of the file -------

store_pointer	lfsr	FSR0,H'220'
		movf	v_m_add0,0
		movwf	POSTINC0
		movf	v_m_add1,0
		movwf	POSTINC0
		movf	v_m_add2,0
		movwf	POSTINC0

		return

;-- restore the pointer to the start of the file -------

restore_pointer	lfsr	FSR0,H'220'
		movf	POSTINC0,0
		movwf	v_m_add0
		movf	POSTINC0,0
		movwf	v_m_add1
		movf	POSTINC0,0
		movwf	v_m_add2
		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Close the File <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
file_close	lfsr	FSR0, H'200'
		movlw	H'04'			;End of File
		movwf	POSTINC0
		movf	v_file,0
		movwf	POSTINC0
		movlw	H'FF'
		movwf	POSTINC0		;(02)
		call	inc_rec_no
		movf	v_rec_lsb,0
		movwf	POSTINC0		;(03)
		movf	v_rec_msb,0
		movwf	POSTINC0		;(04)

		movlw	H'FF'
		movwf	POSTINC0		;(05)
		movwf	POSTINC0		;(06)
		movwf	POSTINC0		;(07)
		movwf	POSTINC0		;(08)
		movwf	POSTINC0		;(09)

		call	s_fill_time2

		call	s_rec_write
		call	s_inc_add16		;set pointer to the next free record

; -------------------------------------------------
; update the first record of the file
; -------------------------------------------------
		call	restore_pointer

		call	s_rec_read
		lfsr	FSR0,H'202'
		movlw	H'01'
		movwf	POSTINC0
		movf	v_rec_lsb,0
		movwf	POSTINC0		;(03)
		movf	v_rec_msb,0
		movwf	POSTINC0		;(04)
		call	s_rec_write
		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>    Insert temperature record    <<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
file_insert01	lfsr	FSR0, H'200'
		movlw	H'01'			;Temperature record
		movwf	POSTINC0
		movf	v_file,0
		movwf	POSTINC0		;(01) -- File ID
		movlw	H'FF'
		movwf	POSTINC0		;(02)

		call	inc_rec_no
		movf	v_rec_lsb,0
		movwf	POSTINC0		;(03)
		movf	v_rec_msb,0
		movwf	POSTINC0		;(04)

		movf	v_i1,0
		movwf	POSTINC0		;(05) - Temperature Sensor 1
		movf	v_i2,0
		movwf	POSTINC0		;(06) - Temperature Sensor 2
		movf	v_i3,0
		movwf	POSTINC0		;(07) - Temperature Sensor 3
		movf	v_i4,0
		movwf	POSTINC0		;(08) - Temperature Sensor 4

		movlw	H'FF'
		movwf	POSTINC0		;(09)

		call	s_fill_time2

		call	s_rec_write
		call	s_inc_add16		;set pointer to the next free record

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_rec_no	incf	v_rec_lsb,1
		btfss	STATUS,Z
		return
		incf	v_rec_msb,1
		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Store File Info <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;temporary storage of currently opened file
;   the reason is, that other file can be opened, but
;   later it is important to "return to the current file"

;file info is stored in PIC memory from: 230h

;following variables are stored:
;   v_num_files
;   v_file
;   v_m_add0
;   v_m_add1
;   v_m_add2
;   v_rec_lsb
;   v_rec_msb
;
;variables at 220h are not stored => other file can be opened only in read mode
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
store_curr_fi	lfsr	FSR0, H'230'
		movf	v_num_files,0
		movwf	POSTINC0
		movf	v_file,0
		movwf	POSTINC0
		movf	v_m_add0,0
		movwf	POSTINC0
		movf	v_m_add1,0
		movwf	POSTINC0
		movf	v_m_add2,0
		movwf	POSTINC0
		movf	v_rec_lsb,0
		movwf	POSTINC0
		movf	v_rec_msb,0
		movwf	POSTINC0

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;>>>>>>>>>>>>>>>>>>>>>>>>>>  Resotre File Info <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;file info is restored from PIC memory from: 230h

;following variables are restored:
;   v_num_files
;   v_file
;   v_m_add0
;   v_m_add1
;   v_m_add2
;   v_rec_lsb
;   v_rec_msb
;
;variables at 220h are not touched
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
restore_curr_fi	lfsr	FSR0, H'230'
		movf	POSTINC0,0
		movwf	v_num_files
		movf	POSTINC0,0
		movwf	v_file
		movf	POSTINC0,0
		movwf	v_m_add0
		movf	POSTINC0,0
		movwf	v_m_add1
		movf	POSTINC0,0
		movwf	v_m_add2
		movf	POSTINC0,0
		movwf	v_rec_lsb
		movf	POSTINC0,0
		movwf	v_rec_msb

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Loop / Main Screen
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

;------------------------------------------------------------------------------
;preparation
tl_ms01_p	call	s_control_rec
tl_ms01_p2	call	check_files
		clrf	v_dic
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
tl_ms01		call	disp_clr1
		call	disp_clr2

;----------------- RTC get data -----------------------
		movlw	B'11010000'		;RTC = 1101000x
		movwf	v_i2c_add

tl_ms03		call	d2
tl_ms02		call	rtc_i2c_read
		call	rtc_translate			;translate RTC date from RTC device to display format

;Second 0 - execute meassurement
		movf	v_rtc_s2,0
		xorlw	'0'
		btfsc	STATUS,Z
		bsf	v_md_status,D'007'		;initialize temperature conversion

		btfss	v_md_status,D'006'		;conversion running
		call	ds1820_m_start

;Second 2 - read sensors
		movf	v_rtc_s2,0
		xorlw	'2'
		btfsc	STATUS,Z
		call	read_sensors

;Second 3 - write temperature record, if required
		movf	v_rtc_s2,0
		xorlw	'3'
		btfsc	STATUS,Z
		call	store_values

		call	w_ms001
;------------------------------------------------------
;keaboard test ----------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

		btfss	v_md_status,D'000'
		goto	tl_ms03

uwaitclk	btfss	PORTB,D'007'
		goto	uwaitclk

uwaitdat	btfss	PORTB,D'006'
		goto	uwaitdat

uwaitclk2	btfsc	PORTB,D'007'
		goto	uwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'3A'			;M - Menu
		btfsc	STATUS,Z
		goto	lg_main_menu

		goto	tl_ms02


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Memory List
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

memory_list2	bsf	v_md_status2,D'005'		;current file only
		bcf	v_md_status2,D'004'

		goto	ml007

;------------------------------------------------------------------------------
memory_list	call	disp_clr1
		call	disp_clr2

		clrf	v_m_add0
		clrf	v_m_add1
		clrf	v_m_add2

		call	open_for_read

ml004		call	s_rec_next
;;;;;;;		call	w_buffer_hex
ml007		call	w_f_info


ml002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER = Next
		btfsc	STATUS,Z
		goto	ml003


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	ml005

		goto	ml002

;------------------------------------------------------------------------------
ml003		call	dr2
		btfsc	v_md_status2,D'004'
		goto	ml005_2			;exit for single file display after EOF record
		goto	ml004
;------------------------------------------------------------------------------
ml005		btfsc	v_md_status2,D'005'
		goto	ml005_2
		call	spi_cs0_dis
		call	check_files
		goto	m_files
;------------------------------------------------------------------------------
ml005_2		call	disp_clr1
		call	disp_clr2
		bcf	v_md_status2,D'005'	;single file mode
		bcf	v_md_status2,D'004'	;EOF reached
		goto	m_fem06
;------------------------------------------------------------------------------
w_buffer_hex	movlw	B'10000000'
		call	send_comm

		lfsr	FSR0, H'200'
		call	w_hex_row

		movlw	B'11000000'
		call	send_comm

		call	w_hex_row

		return
;------------------------------------------------------------------------------
w_hex_row	movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex

		return



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Memory Information
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_info		call	disp_clr1
		call	disp_clr2

		btfsc	v_md_status,D'002'
		goto	mei003
		btfsc	v_md_status,D'003'
		goto	mei003
		btfsc	v_md_status,D'004'
		goto	mei003
		goto	m_memory

mei003		call	check_files
		call	disp_clr1
		call	disp_clr2

		call	w_m_info

mei002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER = back to main menu
		btfsc	STATUS,Z
		goto	m_memory

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main menu
		btfsc	STATUS,Z
		goto	m_memory

		goto	mei002

;------------------------------------------------------------------------------
w_m_info_full	movlw	high t_memf1b
		movwf	TBLPTRH
		movlw	low t_memf1b
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
w_m_info	movlw	B'10000000'
		call	send_comm

		btfsc	v_md_status2,D'000'
		goto	w_m_info_full

		call	wm_U
		call	wm_s
		call	wm_e
		call	wm_d
		call	wm_dvojbodka
		call	wm_space
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_0
		call	wm_minus
		call	s_dec_add1i
		movf	v_m_add2,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add1,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add0,0
		movwf	kbd_buffer
		call	write_hex

		call	s_inc_add1i
		movlw	B'11000000'
		call	send_comm
		call	wm_F
		call	wm_r
		call	wm_e
		call	wm_e
		call	wm_dvojbodka
		call	wm_space
		movf	v_m_add2,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add1,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add0,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus
		btfss	v_md_status,D'004'
		goto	w_e64k
		call	wm_0
		call	wm_F
		call	wm_F
		call	wm_F
		call	wm_F
		call	wm_F
		return
;------------------------------------------------------------------------------
w_e64k		call	wm_0
		call	wm_0
		call	wm_F
		call	wm_F
		call	wm_F
		call	wm_F
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; Erase memory
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_erase_exec	btfsc	v_md_status,D'002'
		goto	erase_nvram

		btfsc	v_md_status,D'003'
		goto	erase_eeprom

		btfsc	v_md_status,D'004'
		goto	erase_flash

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;for NVRAM it is enough to "destroy" first byte of the control record
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
erase_nvram	call	spi_cs0_en
		movlw	H'02'			;Write
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		call	spi_cs0_dis

		reset				;RESET
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;EEPROM - we have to execute CE (Chip Erase) command
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
erase_eeprom	call	ee_wren
		call	spi_cs0_en
		movlw	B'11000111'		;Chip Erase
		call	spi_send
		call	spi_cs0_dis
		call	dr2

		reset				;RESET
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;Flash - the same procedure as for EEPROM
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
erase_flash	goto	erase_eeprom
;------------------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Memory Erase
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_erase		call	disp_clr1
		call	disp_clr2
		call	w_erase

mer002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER = Erase
		btfsc	STATUS,Z
		goto	m_erase_exec

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main menu
		btfsc	STATUS,Z
		goto	m_memory

		goto	mer002

;------------------------------------------------------------------------------
w_erase		movlw	B'10000000'
		call	send_comm
		movlw	high t_erase1
		movwf	TBLPTRH
		movlw	low t_erase1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'
		call	send_comm
		movlw	high t_erase2
		movwf	TBLPTRH
		movlw	low t_erase2
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Memory Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_memory	call	disp_clr1
		call	disp_clr2
		call	w_memory

mem002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = Info
		btfsc	STATUS,Z
		goto	m_info

		movf	kbd_buffer,0
		xorlw	H'24'			;E = Erase
		btfsc	STATUS,Z
		goto	m_erase

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main menu
		btfsc	STATUS,Z
		goto	lg_system_menu

		goto	mem002

;------------------------------------------------------------------------------
w_memory	movlw	B'10000000'
		call	send_comm

		call	wm_T
		call	wm_y
		call	wm_p
		call	wm_e
		call	wm_dvojbodka
		call	wm_space

		btfsc	v_md_status,D'002'
		goto	w_m_nvram
		btfsc	v_md_status,D'003'
		goto	w_m_eeprom
		btfsc	v_md_status,D'004'
		goto	w_m_flash

		call	wm_N
		call	wm_o
		call	wm_space
		call	wm_M
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_r
		call	wm_y

		return
;------------------------------------------------------------------------------
w_m_nvram	call	wm_N
		call	wm_V
		call	wm_R
		call	wm_A
		call	wm_M
		call	wm_space
		call	wm_6
		call	wm_4
		call	wm_k
		call	wm_B
		goto	w_memory2

w_m_eeprom	call	wm_E
		call	wm_E
		call	wm_P
		call	wm_R
		call	wm_O
		call	wm_M
		call	wm_space
		call	wm_6
		call	wm_4
		call	wm_k
		call	wm_B
		goto	w_memory2

w_m_flash	call	wm_F
		call	wm_l
		call	wm_a
		call	wm_s
		call	wm_h
		call	wm_space
		call	wm_1
		call	wm_M
		call	wm_B

;------------------------------------------------------------------------------
w_memory2	movlw	B'11000000'
		call	send_comm

		movlw	high t_mem2
		movwf	TBLPTRH
		movlw	low t_mem2
		movwf	TBLPTRL
		call	w_send_txt
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

lg_main_menu	call	disp_clr1
		call	disp_clr2
		call	w_lgm001

lgm002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = System
		btfsc	STATUS,Z
		goto	lg_system_menu

		movf	kbd_buffer,0
		xorlw	H'2B'			;F = Files
		btfsc	STATUS,Z
		goto	m_files_st

		movf	kbd_buffer,0
		xorlw	H'22'			;X = SPI Editor
		btfsc	STATUS,Z
		goto	spi_editor

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	tl_ms01

		goto	lgm002
;------------------------------------------------------------------------------
m_files_st	btfsc	v_md_status,D'002'
		goto	m_files
		btfsc	v_md_status,D'003'
		goto	m_files
		btfsc	v_md_status,D'004'
		goto	m_files
;no memory installed
		goto	lg_main_menu

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Menu --> System
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

lg_system_menu	call	disp_clr1
		call	disp_clr2
		call	w_lgsm001

lgsm002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = Software Reset
		btfsc	STATUS,Z
		reset

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = Sensors Menu
		btfsc	STATUS,Z
		goto	m_sensors

		movf	kbd_buffer,0
		xorlw	H'21'			;C = Clock
		btfsc	STATUS,Z
		goto	rtc_set_p

		movf	kbd_buffer,0
		xorlw	H'3A'			;M = Memory
		btfsc	STATUS,Z
		goto	m_memory


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main menu
		btfsc	STATUS,Z
		goto	lg_main_menu

		goto	lgsm002

;------------------------------------------------------------------------------
rtc_set_p	call	disp_clr1
		call	disp_clr2

		call	rtc_i2c_read
		call	rtc_translate

		goto	rtc_set
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_lgm001	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_lgmm1
		movwf	TBLPTRH
		movlw	low t_lgmm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_lgmm2
		movwf	TBLPTRH
		movlw	low t_lgmm2
		movwf	TBLPTRL
		call	w_send_txt
		return

;------------------------------------------------------------------------------
w_lgsm001	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_lgsm1
		movwf	TBLPTRH
		movlw	low t_lgsm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_lgsm2
		movwf	TBLPTRH
		movlw	low t_lgsm2
		movwf	TBLPTRL
		call	w_send_txt
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File in progress menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_file_3	call	disp_clr1
		call	disp_clr2


		call	w_fip

ffip1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = Stop
		btfsc	STATUS,Z
		goto	m_file_stop


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	lg_main_menu

		goto	ffip1

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

m_file_stop	call	file_close
		goto	tl_ms01_p2

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_fip		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_fip1
		movwf	TBLPTRH
		movlw	low t_fip1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_fip2
		movwf	TBLPTRH
		movlw	low t_fip2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'10010001'		;row: 1 / col: 17
		call	send_comm
		movf	v_file,0
		movwf	kbd_buffer
		call	write_hex

		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File Information
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;pointer to the first record of the file needs to be stored at 220h


m_file_info	movf	v_file,0
		btfsc	STATUS,Z
		goto	m_files			;no file selected

		call	store_curr_fi		;store information about current opened file
		call	restore_pointer		;get the begining of the file (from 220h)

;---------------------------------------------
		call	open_for_read
m_fi_next	call	s_rec_next		;now we have a record in the buffer (from 200h)
;---------------------------------------------

		call	disp_clr1
		call	disp_clr2

		call	w_f_info

finf1		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER = Next record
		btfsc	STATUS,Z
		goto	m_fi_next


		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	m_fi_escape

		goto	finf1


m_fi_escape	call	restore_curr_fi
		goto	m_files


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Display Record Information
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
w_f_info	call	disp_clr1
		call	disp_clr2
		bcf	v_md_status2,D'004'

w_f_info2	movlw	B'10000000'
		call	send_comm

		lfsr	FSR0, H'200'

		movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_fi_empty

		movf	INDF0,0
		xorlw	H'03'			;Start of File
		btfsc	STATUS,Z
		goto	w_fi_03

		movf	INDF0,0
		xorlw	H'04'			;End of File
		btfsc	STATUS,Z
		goto	w_fi_04

		movf	INDF0,0
		xorlw	H'01'			;Temperature Record
		btfsc	STATUS,Z
		goto	w_fi_01

		movf	INDF0,0
		xorlw	H'AA'			;Memory Control Record
		btfsc	STATUS,Z
		goto	w_fi_AA


		goto	w_fi_unkn

;-----------------------------------------------------------------------------
w_fi_AA		movlw	high t_cc1
		movwf	TBLPTRH
		movlw	low t_cc1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'
		call	send_comm

		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt

		movlw	B'11011000'
		call	send_comm

		return
;-----------------------------------------------------------------------------
w_fi_01		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt

		call	wm_minus

		lfsr	FSR0, H'204'
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex		;Record ID - MSB
		movf	INDF0,0
		movwf	kbd_buffer
		call	write_hex		;Record ID - LSB

		movlw	B'11000000'
		call	send_comm

		lfsr	FSR0, H'205'
		movf	POSTINC0,0
		movwf	v_i1
		movf	POSTINC0,0
		movwf	v_i2
		movf	POSTINC0,0
		movwf	v_i3
		movf	POSTINC0,0
		movwf	v_i4

		call	wm_A
		call	wm_dvojbodka
		call	wm_tempA
		call	wm_space

		call	wm_B
		call	wm_dvojbodka
		call	wm_tempB
		call	wm_space

		call	wm_C
		call	wm_dvojbodka
		call	wm_tempC
		call	wm_space

		call	wm_D
		call	wm_dvojbodka
		call	wm_tempD
		call	wm_space

		movlw	B'11011000'
		call	send_comm

		return
;-----------------------------------------------------------------------------
w_fi_03		call	wm_I
		call	wm_D
		call	wm_dvojbodka
		lfsr	FSR0, H'201'
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex		;File ID

;--------------------------------------
		movlw	B'11010110'
		call	send_comm
		movf	INDF0,0
		xorlw	H'FF'
		btfss	STATUS,Z
		call	wm_C			;File is closed

		movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		call	wm_R			;File is running
;--------------------------------------
		movlw	B'10000101'
		call	send_comm

		call	wm_minus
		call	wm_minus
		call	wm_R
		call	wm_e
		call	wm_c
		call	wm_s
		call	wm_dvojbodka
		lfsr	FSR0, H'204'
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex		;File Length - MSB
		movf	INDF0,0
		movwf	kbd_buffer
		call	write_hex		;File Length - LSB

		call	wm_minus
		call	wm_minus
		call	wm_I
		call	wm_dvojbodka
		lfsr	FSR0, H'206'
		movf	INDF0,0
		movwf	v_interval
		call	w_interval


		movlw	B'11000000'
		call	send_comm

		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt

		movlw	B'11011000'
		call	send_comm

		return
;-----------------------------------------------------------------------------
w_fi_04		btfsc	v_md_status2,D'005'
		bsf	v_md_status2,D'004'

		call	wm_I
		call	wm_D
		call	wm_dvojbodka
		lfsr	FSR0, H'201'
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex		;File ID

		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_F
		call	wm_i
		call	wm_n
		call	wm_i
		call	wm_s
		call	wm_h
		call	wm_e
		call	wm_d

		movlw	B'11000000'
		call	send_comm

		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt

		movlw	B'11011000'
		call	send_comm

		return
;-----------------------------------------------------------------------------
w_fi_unkn	movlw	high t_fi_rXX
		movwf	TBLPTRH
		movlw	low t_fi_rXX
		movwf	TBLPTRL
		call	w_send_txt
		return
;-----------------------------------------------------------------------------
w_fi_empty	movlw	high t_fi_rFF
		movwf	TBLPTRH
		movlw	low t_fi_rFF
		movwf	TBLPTRL
		call	w_send_txt
		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      USB Transfer 1
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------
;-------------------------------------------------------------------
usb_t_header	call	disp_clr1
		call	disp_clr2

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_usbti1
		movwf	TBLPTRH
		movlw	low t_usbti1
		movwf	TBLPTRL
		call	w_send_txt
;-------------------------------------------------------------------

		call	open_usb

		movlw	high t_usb_h1
		movwf	TBLPTRH
		movlw	low t_usb_h1
		movwf	TBLPTRL
		call	w_send_usb

		call	w_usb_nl
		call	w_usb_nl
		return
;-------------------------------------------------------------------
;-------------------------------------------------------------------
usb_transfer1	call	usb_t_header

		btfsc	v_md_status2,D'005'
		goto	usb005			;current file only

		clrf	v_m_add0
		clrf	v_m_add1
		clrf	v_m_add2

		movlw	H'FF'
		movwf	v_rec_lsb
		movwf	v_rec_msb

		call	open_for_read

usbt1_2		call	s_rec_next

usb005		lfsr	FSR0, H'200'		;start of the record buffer

		movf	INDF0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	usb_t1_fin

;load the temperature variables -----------
		lfsr	FSR0, H'205'
		movf	POSTINC0,0
		movwf	v_i1
		movf	POSTINC0,0
		movwf	v_i2
		movf	POSTINC0,0
		movwf	v_i3
		movf	POSTINC0,0
		movwf	v_i4
;-----------------------------------------
		call	w_usb_t_inf
		call	w_usb_record

		call	s_inc_add16i
;------------------------------------------------------
;------------------------------------------------------
;keaboard test ----------------------------------------
;------------------------------------------------------
		movlw	H'FE'
		movwf	kbd_out
		call	send_ck

jwaitclk	btfss	PORTB,D'007'
		goto	jwaitclk

jwaitdat	btfss	PORTB,D'006'
		goto	jwaitdat

jwaitclk2	btfsc	PORTB,D'007'
		goto	jwaitclk2

		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to file menu
		btfsc	STATUS,Z
		goto	usb_t1_fin2


		btfsc	v_md_status2,D'004'
		goto	usb_t1_fin3			;exit for single file display after EOF record

;------------------------------------------------------
;------------------------------------------------------
;------------------------------------------------------
		goto	usbt1_2
;------------------------------------------------------
usb_t1_fin	call	spi_cs0_dis
		call	close_usb

		call	check_files

		goto	m_files

;-----------------------------------------------------------------------------
usb_t1_fin2	movlw	high t_usb_irr
		movwf	TBLPTRH
		movlw	low t_usb_irr
		movwf	TBLPTRL
		call	w_send_usb

		btfsc	v_md_status2,D'005'
		goto	usb_t1_fin3

		goto	usb_t1_fin
;-----------------------------------------------------------------------------
usb_t1_fin3	call	disp_clr1
		call	disp_clr2
		bcf	v_md_status2,D'005'	;single file mode
		bcf	v_md_status2,D'004'	;EOF reached

		call	close_usb

		goto	m_fem06
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
open_usb	call	i2c_start
		movlw	B'01100000'
		call	i2c_send
		return
;-----------------------------------------------------------------------------
close_usb	call	i2c_stop
		return
;-----------------------------------------------------------------------------
w_send_usb	tblrd*+
		movf	TABLAT,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		return

		movf	TABLAT,0
		call	i2c2usb
		goto	w_send_usb

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c2usb		call	i2c_send
		call	dr56
		return
;-----------------------------------------------------------------------------
dr56		MOVLW   D'009'
		MOVWF   TIMER2
DEL_LOOPb1q	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPb2q	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2q
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1q
		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;send a record from buffer thorugh USB
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_usb_record	lfsr	FSR0, H'200'		;start of the record buffer

		btfsc	v_md_status2,D'006'
		goto	w_usb_hex

		movf	INDF0,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	w_usb_01

		movf	INDF0,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	w_usb_03

		movf	INDF0,0
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	w_usb_04

		goto	w_usb_oth
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_usb_minus	movlw	high t_usb_minus
		movwf	TBLPTRH
		movlw	low t_usb_minus
		movwf	TBLPTRL
		call	w_send_usb

		call	w_usb_nl

		return
;-----------------------------------------------------------------------------
w_usb_nl	movlw	D'013'
		call	i2c2usb
		return
;-----------------------------------------------------------------------------
;End of file
w_usb_04	call	w_usb_nl

		movlw	high t_usb_04
		movwf	TBLPTRH
		movlw	low t_usb_04
		movwf	TBLPTRL
		call	w_send_usb

		movf	POSTINC0,0
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex_usb


		movlw	' '
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		movlw	' '
		call	i2c2usb

;send time & date

		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt_usb

		call	w_usb_nl
		call	w_usb_minus

		btfsc	v_md_status2,D'005'
		bsf	v_md_status2,D'004'

		return

;-----------------------------------------------------------------------------
;Start of file
w_usb_03	call	w_usb_minus

		movlw	high t_usb_03
		movwf	TBLPTRH
		movlw	low t_usb_03
		movwf	TBLPTRL
		call	w_send_usb

		movf	POSTINC0,0
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex_usb

		movlw	' '
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		movlw	' '
		call	i2c2usb

;send time & date

		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt_usb

		call	w_usb_nl
		call	w_usb_nl

		return
;-----------------------------------------------------------------------------
;Temperature record
w_usb_01	nop

;send admin info

		btfss	v_md_status2,D'007'
		goto	wusb1_1
		movlw	'R'
		call	i2c2usb
		movlw	':'
		call	i2c2usb

		movf	POSTINC0,0
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex_usb

		movlw	'/'
		call	i2c2usb

		lfsr	FSR0, H'204'
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex_usb
		movf	INDF0,0
		movwf	kbd_buffer
		call	write_hex_usb

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb

		movlw	high t_usb_01
		movwf	TBLPTRH
		movlw	low t_usb_01
		movwf	TBLPTRL
		call	w_send_usb

;send time & date

wusb1_1		call	s_load_time
		call	rtc_translate
		call	w_rtcm_dt_usb

		bsf	v_usb_sw,D'000'
;-----------------------------------------------------------------------------
		btfss	v_md_status2,D'007'
		goto	wusb1_2
;send temperature data - full format

		movlw	' '
		call	i2c2usb
		movlw	'>'
		call	i2c2usb
		movlw	'>'
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'A'
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		call	wm_tempA

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'B'
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		call	wm_tempB

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'C'
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		call	wm_tempC

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		movlw	'D'
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		call	wm_tempD

		bcf	v_usb_sw,D'000'


		movlw	D'013'
		call	i2c2usb

		return
;-----------------------------------------------------------------------------
;send temperature data - slim format
wusb1_2		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		call	wm_tempA

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		call	wm_tempB

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		call	wm_tempC

		movlw	' '
		call	i2c2usb
		movlw	' '
		call	i2c2usb
		call	wm_tempD

		bcf	v_usb_sw,D'000'


		movlw	D'013'
		call	i2c2usb

		return
;-----------------------------------------------------------------------------
w_usb_oth	movlw	high t_usb_oth
		movwf	TBLPTRH
		movlw	low t_usb_oth
		movwf	TBLPTRL
		call	w_send_usb

		call	w_usb_nl

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_usb_hex	movf	v_m_add2,0
		movwf	kbd_buffer
		call	write_hex_usb
		movf	v_m_add1,0
		movwf	kbd_buffer
		call	write_hex_usb
		movf	v_m_add0,0
		movwf	kbd_buffer
		call	write_hex_usb

		movlw	':'
		call	i2c2usb
		movlw	' '
		call	i2c2usb

		movlw	D'015'
		movwf	c_tmp

		lfsr	FSR0, H'200'		;start of the record buffer

		btfsc	v_md_status2,D'005'
		call	check_set_EOF

wux_loop1	movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex_usb
		movlw	'-'
		call	i2c2usb
		decfsz	c_tmp,1
		goto	wux_loop1

		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex_usb

		call	w_usb_nl

		return
;-----------------------------------------------------------------------------
check_set_EOF	movf	INDF0,0
		xorlw	H'04'
		btfsc	STATUS,Z
		bsf	v_md_status2,D'004'
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;display infor about the current USB transfer
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_usb_t_inf	movlw	B'10000000'
		call	send_comm

		lfsr	FSR0, H'200'

		movf	INDF0,0
		xorlw	H'01'
		btfss	STATUS,Z
		goto	w_usb_t_infc1

		movf	POSTINC0,0
		call	wm_F
		call	wm_dvojbodka
		movf	POSTINC0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
		call	wm_R
		call	wm_dvojbodka

		lfsr	FSR0, H'204'
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex
		movf	INDF0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_sls
		movf	v_rec_msb,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_rec_lsb,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
		call	wm_space

		movf	v_m_add2,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add1,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add0,0
		movwf	kbd_buffer
		call	write_hex

		return
;-----------------------------------------------------------------------------
w_usb_t_infc1	call	disp_clr1

		movf	INDF0,0
		xorlw	H'03'
		btfss	STATUS,Z
		return

		lfsr	FSR0, H'203'
		movf	POSTINC0,0
		movwf	v_rec_lsb
		movf	POSTINC0,0
		movwf	v_rec_msb

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      USB Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_usb2		bsf	v_md_status2,D'005'
		bcf	v_md_status2,D'004'
;---------------------------------------------
m_usb		call	disp_clr1
		call	disp_clr2

		call	w_usb_m

musb01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'16'			;1 = Full Format
		btfsc	STATUS,Z
		goto	m_usb_full

		movf	kbd_buffer,0
		xorlw	H'1E'			;2 = Slim Format
		btfsc	STATUS,Z
		goto	m_usb_slim

		movf	kbd_buffer,0
		xorlw	H'33'			;H = Hex format
		btfsc	STATUS,Z
		goto	m_usb_hex

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfss	STATUS,Z
		goto	musb01

		clrf	v_file
		goto	m_files
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_usb_exit	btfss	v_md_status2,D'005'
		goto	m_files
		goto	m_fem06
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_usb_full	bsf	v_md_status2,D'007'
		bcf	v_md_status2,D'006'
		goto	usb_transfer1
;------------------------------------------------------------------------------
m_usb_slim	bcf	v_md_status2,D'007'
		bcf	v_md_status2,D'006'
		goto	usb_transfer1
;------------------------------------------------------------------------------
m_usb_hex	bcf	v_md_status2,D'007'
		bsf	v_md_status2,D'006'
		goto	usb_transfer1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_usb_m		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_usbm1
		movwf	TBLPTRH
		movlw	low t_usbm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_usbm2
		movwf	TBLPTRH
		movlw	low t_usbm2
		movwf	TBLPTRL
		call	w_send_txt

		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File Entry Menu - File List
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_fem		call	disp_clr1
		call	disp_clr2

		movlw	D'001'
		movwf	v_file
		call	file_open
		movf	v_file,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	m_fem_exit		;no files exist --> exit
		

m_fem07		clrf	v_file

m_fem04		incf	v_file,1
m_fem06		call	file_open

		movf	v_file,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	m_fem07			;file not found -- start from file #01

		call	w_fem

m_fem03		clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'2D'			;R = Read
		btfsc	STATUS,Z
		goto	memory_list2

		movf	kbd_buffer,0
		xorlw	H'3C'			;U = USB
		btfsc	STATUS,Z
		goto	m_usb2

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER = next file
		btfsc	STATUS,Z
		goto	m_fem04

		movf	kbd_buffer,0
		xorlw	H'74'			;-> = next file
		btfsc	STATUS,Z
		goto	m_fem04

		movf	kbd_buffer,0
		xorlw	H'6B'			;<- = previous file
		btfsc	STATUS,Z
		goto	m_fem05

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	m_fem_exit

		goto	m_fem03
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_fem05		decf	v_file,1
		movf	v_file,0
		btfsc	STATUS,Z
		goto	m_fem04
		goto	m_fem06
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_fem_exit	call	check_files
		bcf	v_md_status2,D'005'
		bcf	v_md_status2,D'004'
		goto	m_files
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_fem		movlw	B'10000000'
		call	send_comm

		call	wm_I
		call	wm_D
		call	wm_dvojbodka

		lfsr	FSR0, H'201'
		movf	INDF0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space

		lfsr	FSR0, H'206'
		movf	INDF0,0
		movwf	v_interval
		call	w_interval

		call	wm_space
		call	wm_minus
		call	wm_minus
		call	wm_space

		call	wm_R
		call	wm_dvojbodka

		lfsr	FSR0, H'204'
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex
		movf	POSTDEC0,0
		movwf	kbd_buffer
		call	write_hex


		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_fem2
		movwf	TBLPTRH
		movlw	low t_fem2
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File Main Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_files		call	disp_clr1
		call	disp_clr2

		bcf	v_md_status2,D'005'

		movf	v_file,0
		xorlw	H'00'
		btfss	STATUS,Z
		goto	m_file_3		;recording in progress

		call	w_files

fmm01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'31'			;N = New File
		btfsc	STATUS,Z
		goto	f_new_file

		movf	kbd_buffer,0
		xorlw	H'4B'			;L = List
		btfsc	STATUS,Z
		goto	m_fem

		movf	kbd_buffer,0
		xorlw	H'2D'			;R = Read All
		btfsc	STATUS,Z
		goto	memory_list

		movf	kbd_buffer,0
		xorlw	H'3C'			;U = USB All
		btfsc	STATUS,Z
		goto	m_usb

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	lg_main_menu

		goto	fmm01
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_files		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_file1
		movwf	TBLPTRH
		movlw	low t_file1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_file2
		movwf	TBLPTRH
		movlw	low t_file2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11010010'		;row: 2 / col: 18
		call	send_comm
		movlw	'['
		call	send_data

		movf	v_num_files,0
		movwf	kbd_buffer
		call	write_dec

		movlw	']'
		call	send_data

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      File - New
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
f_new_file	call	disp_clr1
		call	disp_clr2

		btfsc	v_md_status2,D'000'
		goto	m_files

f_new02		movlw	H'01'
		movwf	v_interval

f_new03		call	w_new_file

f_new01		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1B'			;S = Start
		btfsc	STATUS,Z
		goto	fnf_start

		movf	kbd_buffer,0
		xorlw	H'4D'			;P = Period
		btfsc	STATUS,Z
		goto	fnf_period

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	m_files

		goto	f_new01
;------------------------------------------------------------------------------
fnf_period	call	dr2
		incf	v_interval,1
		movf	v_interval,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	f_new02
		goto	f_new03
;------------------------------------------------------------------------------
fnf_start	call	file_create
		goto	tl_ms01

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_new_file	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_new_file1
		movwf	TBLPTRH
		movlw	low t_new_file1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_new_file2
		movwf	TBLPTRH
		movlw	low t_new_file2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'10001001'		;row: 1 / col: 9
		call	send_comm
		movf	v_num_files,0
		movwf	kbd_buffer
		incf	kbd_buffer,1
		call	write_hex

		movlw	B'11001001'		;row: 2 / col: 9
		call	send_comm
		call	w_interval
		return

;------------------------------------------------------------------------------
w_interval	movf	v_interval,0
		xorlw	H'01'
		btfsc	STATUS,Z
		call	w_int10s

		movf	v_interval,0
		xorlw	H'02'
		btfsc	STATUS,Z
		call	w_int1m

		movf	v_interval,0
		xorlw	H'03'
		btfsc	STATUS,Z
		call	w_int10m

		movf	v_interval,0
		xorlw	H'04'
		btfsc	STATUS,Z
		call	w_int1h

		return

;------------------------------------------------------------------------------
w_int10s	call	wm_1
		call	wm_0
		call	wm_s
		return
;------------------------------------------------------------------------------
w_int1m		call	wm_1
		call	wm_space
		call	wm_m
		return
;------------------------------------------------------------------------------
w_int10m	call	wm_1
		call	wm_0
		call	wm_m
		return
;------------------------------------------------------------------------------
w_int1h		call	wm_1
		call	wm_space
		call	wm_h
		return
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Sensors Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_sensors	call	disp_clr1
		call	disp_clr2
		call	w_sm001

		clrf	v_tmp1

sen_m002	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = Device Info
		btfsc	STATUS,Z
		goto	m_device_info

		movf	kbd_buffer,0
		xorlw	H'23'			;D = Deregistration
		btfsc	STATUS,Z
		goto	m_s_dereg

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to main loop
		btfsc	STATUS,Z
		goto	lg_system_menu

		goto	sen_m002

;------------------------------------------------------------------------------
m_s_dereg	bsf	v_tmp1,D'000'		;deregistration
		goto	m_device_info
;------------------------------------------------------------------------------
w_sm001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_sm1
		movwf	TBLPTRH
		movlw	low t_sm1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_sm2
		movwf	TBLPTRH
		movlw	low t_sm2
		movwf	TBLPTRL
		call	w_send_txt
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Device Info
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

m_device_info	nop

qowms3		clrf	v_ow_curr
		clrf	ee_add
;--------------------------------------------
qowms2		call	disp_clr1
		call	disp_clr2

		movf	ee_add,0
		movwf	v_tmp2			;save the address of the current sensor

		call	load_ROMID_ee
		call	w_sensor_info

		call	dr2

qow_rs_1	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'5A'			;ENTER
		btfsc	STATUS,Z
		goto	qow_xx_next

		movf	kbd_buffer,0
		xorlw	H'23'			;D = Deregister
		btfsc	STATUS,Z
		goto	qow_xx_del

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	m_sensors

		goto	qow_rs_1
;----------------------------------------------------------------------
qow_xx_del	btfss	v_tmp1,D'000'
		goto	qow_rs_1		;we are not in deregistration mode

;-----------------------------------------------------
;no we have just to delete the sensor from PIC EEPROM
		movf	v_tmp2,0
		movwf	ee_add

		movlw	H'FF'
		movwf	ee_buffer

		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte
		incf	ee_add,1
		call	ee_write_byte

		movf	v_tmp2,0
		movwf	ee_add

		movf	v_ow_curr,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bcf	v_sensors,D'000'

		movf	v_ow_curr,0
		xorlw	D'001'
		btfsc	STATUS,Z
		bcf	v_sensors,D'001'

		movf	v_ow_curr,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bcf	v_sensors,D'002'

		movf	v_ow_curr,0
		xorlw	D'003'
		btfsc	STATUS,Z
		bcf	v_sensors,D'003'
;-----------------------------------------------------
		goto	qowms2
;----------------------------------------------------------------------
qow_xx_next	incf	v_ow_curr,1
		movf	v_ow_curr,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	qowms3
;
		goto	qowms2

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_sensor_info	movlw	B'10000000'
		call	send_comm
		movlw	'A'
		addwf	v_ow_curr,0
		call	send_data
		call	wm_dvojbodka
		call	wm_space

		movf	ow_ROM_0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_s_nr		;sensor not registered

		movf	ow_ROM_0,0
		xorlw	H'10'
		btfsc	STATUS,Z
		call	w_ow_DS18S20

		movf	ow_ROM_0,0
		xorlw	H'28'
		btfsc	STATUS,Z
		call	w_ow_DS18B20

		call	wm_space

		call	w_s_active_i

		btfss	v_tmp1,D'000'
		goto	w_ds1820i2
;------------------------------------
;we are in deregistration part

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm
		movlw	high t_sm5
		movwf	TBLPTRH
		movlw	low t_sm5
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
w_s_nr		call	wm_N
		call	wm_o
		call	wm_t
		call	wm_space
		call	wm_r
		call	wm_e
		call	wm_g
		call	wm_i
		call	wm_s
		call	wm_t
		call	wm_e
		call	wm_r
		call	wm_e
		call	wm_d
		return
;------------------------------------------------------------------------------
w_s_active_i	movf	v_ow_curr,0
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	w_s_actA

		movf	v_ow_curr,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	w_s_actB

		movf	v_ow_curr,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	w_s_actC

		movf	v_ow_curr,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	w_s_actD

		return
;------------------------------------------------------------------------------
w_s_actA	btfsc	v_sensors,D'000'
		goto	w_s_Active
		goto	w_s_Regist
;------------------------------------------------------------------------------
w_s_actB	btfsc	v_sensors,D'001'
		goto	w_s_Active
		goto	w_s_Regist
;------------------------------------------------------------------------------
w_s_actC	btfsc	v_sensors,D'002'
		goto	w_s_Active
		goto	w_s_Regist
;------------------------------------------------------------------------------
w_s_actD	btfsc	v_sensors,D'003'
		goto	w_s_Active
		goto	w_s_Regist
;------------------------------------------------------------------------------
w_s_Active	call	wm_A
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_v
		call	wm_e
		return
;------------------------------------------------------------------------------
w_s_Regist	call	wm_R
		call	wm_e
		call	wm_g
		call	wm_i
		call	wm_s
		call	wm_t
		call	wm_e
		call	wm_r
		call	wm_e
		call	wm_d
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;read temperature from available & registered sensors
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
read_sensors	nop
		call	sensorA_read
		call	sensorB_read
		call	sensorC_read
		call	sensorD_read
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;check what sensor IDs are registered already (A-D)
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
check_regs	movlw	H'00'
		movwf	ee_add
		call	ee_read_byte
		movf	ee_buffer,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_sensors,D'004'

		movlw	H'08'
		movwf	ee_add
		call	ee_read_byte
		movf	ee_buffer,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_sensors,D'005'

		movlw	H'10'
		movwf	ee_add
		call	ee_read_byte
		movf	ee_buffer,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_sensors,D'006'

		movlw	H'18'
		movwf	ee_add
		call	ee_read_byte
		movf	ee_buffer,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_sensors,D'007'

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
match_sensor	nop

		clrf	ee_add
		call	match_sensorx
		btfsc	c_tmp,D'000'
		goto	s_match_A

		movlw	H'08'
		movwf	ee_add
		call	match_sensorx
		btfsc	c_tmp,D'000'
		goto	s_match_B

		movlw	H'10'
		movwf	ee_add
		call	match_sensorx
		btfsc	c_tmp,D'000'
		goto	s_match_C

		movlw	H'18'
		movwf	ee_add
		call	match_sensorx
		btfsc	c_tmp,D'000'
		goto	s_match_D

;sensor not registered

		btfsc	v_md_status,D'000'
		call	register_sensor

		btfsc	v_md_status,D'000'
		return

;no registration w/o kbd possible
		call	w_nrwkp

		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
s_match_A	bsf	v_sensors,D'000'
		return

s_match_B	bsf	v_sensors,D'001'
		return

s_match_C	bsf	v_sensors,D'002'
		return

s_match_D	bsf	v_sensors,D'003'
		return

;vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
;------------------------------------------------------------------------------
;Sensor with ROM ID starting at FSR0 is search with the sensors
;stored in PIC EEPROM
;------------------------------------------------------------------------------
match_sensorx	movlw	D'008'
		movwf	v_ttt
		clrf	c_tmp			;no match - flag

		lfsr	FSR0, ow_ROM_0

ms_next_byte	call	ee_read_byte
		movf	POSTINC0,0
		xorwf	ee_buffer,0
		btfss	STATUS,Z
		return				;no match
		incf	ee_add,1

		decfsz	v_ttt,1
		goto	ms_next_byte

		bsf	c_tmp,D'000'		;match
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
load_ROMID_ee	call	ee_read_byte
		movwf	ow_ROM_0
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_1
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_2
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_3
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_4
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_5
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_6
		incf	ee_add,1

		call	ee_read_byte
		movwf	ow_ROM_7
		incf	ee_add,1

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
load_ROMID	movf	POSTINC0,0
		movwf	ow_ROM_0
		movf	POSTINC0,0
		movwf	ow_ROM_1
		movf	POSTINC0,0
		movwf	ow_ROM_2
		movf	POSTINC0,0
		movwf	ow_ROM_3
		movf	POSTINC0,0
		movwf	ow_ROM_4
		movf	POSTINC0,0
		movwf	ow_ROM_5
		movf	POSTINC0,0
		movwf	ow_ROM_6
		movf	POSTINC0,0
		movwf	ow_ROM_7

		movf	ow_ROM_0,0
		xorlw	H'10'
		btfsc	STATUS,Z
		return				;it is DS18S20

		movf	ow_ROM_0,0
		xorlw	H'28'
		btfsc	STATUS,Z
		return				;it is DS18B20

;---------------------------------------------------------------------------
;not supported device found

		call	disp_clr1
		call	disp_clr2

		movlw	D'128'
		call	send_comm
		movlw	high t_usd1
		movwf	TBLPTRH
		movlw	low t_usd1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	D'192'
		call	send_comm
		movlw	high t_usd2
		movwf	TBLPTRH
		movlw	low t_usd2
		movwf	TBLPTRL
		call	w_send_txt

		movf	ow_ROM_0,0
		movwf	kbd_buffer
		call	write_hex

finito2		goto	finito2

;---------------------------------------------------------------------------
w_nrwkp		call	disp_clr1
		call	disp_clr2

		movlw	D'128'
		call	send_comm
		movlw	high t_new_s1
		movwf	TBLPTRH
		movlw	low t_new_s1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	D'192'
		call	send_comm
		movlw	high t_new_s3
		movwf	TBLPTRH
		movlw	low t_new_s3
		movwf	TBLPTRL
		call	w_send_txt

		return
;---------------------------------------------------------------------------

;*****************************************************************************
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; PIC EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;*****************************************************************************
ee_read_byte	nop
		movf	ee_add,0
		movwf	EEADR			;Data Memory
						;Address to read
		bcf	EECON1, CFGS		;Deselect Config space
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, RD		;EE Read
		movf	EEDATA, W		;W = EEDATA
		movwf	ee_buffer
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ee_write_byte	movf	ee_add,0
		movwf	EEADR			;Data Memory address
		movf	ee_buffer,0
		movwf	EEDATA

		bcf	EECON1, CFGS		;Deselect Config space
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, WREN		;EE Write Enable
		movf	EEDATA, W		;W = EEDATL
		bcf	INTCON, GIE		;Disable INTs. (just for sure here...)
		movlw	H'55'
		movwf	EECON2			;Write 55h
		movlw	H'AA'
		movwf	EECON2			;Write AAh
		bsf	EECON1, WR		;Set WR bit to begin write
		bcf	EECON1, WREN		;Disable writes
eewb2		btfsc	EECON1, WR		;Wait for write to complete
		goto	eewb2

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_reg_sens	movlw	'A'
		btfss	v_sensors,D'004'
		movlw	'-'
		call	send_data

		movlw	'B'
		btfss	v_sensors,D'005'
		movlw	'-'
		call	send_data

		movlw	'C'
		btfss	v_sensors,D'006'
		movlw	'-'
		call	send_data

		movlw	'D'
		btfss	v_sensors,D'007'
		movlw	'-'
		call	send_data
		return
;------------------------------------------------------------------------------
w_sens_info	call	disp_clr1
		call	disp_clr2

;Row 1
		movlw	D'128'
		call	send_comm
		movlw	high t_sens_inf1
		movwf	TBLPTRH
		movlw	low t_sens_inf1
		movwf	TBLPTRL
		call	w_send_txt

		call	w_reg_sens
;Row 2
		movlw	D'192'
		call	send_comm
		movlw	high t_sens_inf2
		movwf	TBLPTRH
		movlw	low t_sens_inf2
		movwf	TBLPTRL
		call	w_send_txt

		movlw	'A'
		btfss	v_sensors,D'000'
		movlw	'-'
		call	send_data

		movlw	'B'
		btfss	v_sensors,D'001'
		movlw	'-'
		call	send_data

		movlw	'C'
		btfss	v_sensors,D'002'
		movlw	'-'
		call	send_data

		movlw	'D'
		btfss	v_sensors,D'003'
		movlw	'-'
		call	send_data

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      New sensor registration
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

register_sensor	call	disp_clr1
		call	disp_clr2
		call	w_new_sr

		bsf	v_re_run,D'000'

nsrx2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'1C'			;A
		btfsc	STATUS,Z
		goto	register_A

		movf	kbd_buffer,0
		xorlw	H'32'			;B
		btfsc	STATUS,Z
		goto	register_B

		movf	kbd_buffer,0
		xorlw	H'21'			;C
		btfsc	STATUS,Z
		goto	register_C

		movf	kbd_buffer,0
		xorlw	H'23'			;D
		btfsc	STATUS,Z
		goto	register_D

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	nsrx2

		bcf	v_re_run,D'000'
		return

;-----------------------------------------------------------------------------
register_A	movlw	H'00'
		movwf	ee_add
		goto	register_X
;-----------------------------------------------------------------------------
register_B	movlw	H'08'
		movwf	ee_add
		goto	register_X
;-----------------------------------------------------------------------------
register_C	movlw	H'10'
		movwf	ee_add
		goto	register_X
;-----------------------------------------------------------------------------
register_D	movlw	H'18'
		movwf	ee_add
		goto	register_X
;-----------------------------------------------------------------------------
register_X	lfsr	FSR0, ow_ROM_0
		movlw	D'008'
		movwf	v_ttt

reg_loop1	movf	POSTINC0,0
		movwf	ee_buffer
		call	ee_write_byte
		incf	ee_add,1
		decfsz	v_ttt,1
		goto	reg_loop1

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_new_sr	movlw	D'128'
		call	send_comm
		movlw	high t_new_s1
		movwf	TBLPTRH
		movlw	low t_new_s1
		movwf	TBLPTRL
		call	w_send_txt

		call	wm_zat1
		call	w_reg_sens
		call	wm_zat2

		movlw	D'192'
		call	send_comm
		movlw	high t_new_s2
		movwf	TBLPTRH
		movlw	low t_new_s2
		movwf	TBLPTRL
		call	w_send_txt

		return
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
request_store	movf	v_interval,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	rq_10sec

		movf	v_interval,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	rq_1min

		movf	v_interval,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	rq_10min

		movf	v_interval,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	rq_1hrs

		return
;------------------------------------------------------------------------------
rq_10sec	bcf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
rq_1min		movf	v_rtc_s1,0
		xorlw	'1'
		btfss	STATUS,Z
		return
		bcf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
rq_10min	movf	v_rtc_n2,0
		xorlw	'1'
		btfss	STATUS,Z
		return
		bcf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
rq_1hrs		movf	v_rtc_n1,0
		xorlw	'1'
		btfss	STATUS,Z
		return
		bcf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
store_values	btfss	v_md_status,D'001'
		return					;quit if there is no file in progress

		btfsc	v_md_status,D'005'
		return

;do we have to store the value now?

		movf	v_interval,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	sv_10sec

		movf	v_interval,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	sv_1min

		movf	v_interval,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	sv_10min

		movf	v_interval,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	sv_1hrs

;------------------------------------------------------------------------------
sv_10sec	call	file_insert01
		bsf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
sv_1min		movf	v_rtc_s1,0
		xorlw	'0'
		btfss	STATUS,Z
		return

		call	file_insert01
		bsf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
sv_10min	movf	v_rtc_n2,0
		xorlw	'0'
		btfss	STATUS,Z
		return

		call	file_insert01
		bsf	v_md_status,D'005'
		return
;------------------------------------------------------------------------------
sv_1hrs		movf	v_rtc_n1,0
		xorlw	'0'
		btfss	STATUS,Z
		return

		call	file_insert01
		bsf	v_md_status,D'005'
		return

;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; RTC
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
rtc_v_init	movlw	'-'
		movwf	v_rtc_d1
		movwf	v_rtc_d2
		movwf	v_rtc_m1
		movwf	v_rtc_m2
		movwf	v_rtc_y1
		movwf	v_rtc_y2
		movwf	v_rtc_y3
		movwf	v_rtc_y4
		movwf	v_rtc_h1
		movwf	v_rtc_h2
		movwf	v_rtc_n1
		movwf	v_rtc_n2
		movwf	v_rtc_s1
		movwf	v_rtc_s2

		movwf	v_rtc_t1
		movwf	v_rtc_t2

		return
;------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;	RTC routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_translate	nop
		movf	v_rtc_sec,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	rtc_v_init				;no RTC device connected

;day
		swapf	v_rtc_day,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_d1

		movf	v_rtc_day,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_d2


;month
		swapf	v_rtc_mnt,0
		andlw	B'00000001'
		addlw	'0'
		movwf	v_rtc_m1

		movf	v_rtc_mnt,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_m2

;year
		movlw	'2'
		movwf	v_rtc_y1
		movlw	'0'
		movwf	v_rtc_y2

		swapf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y3

		movf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y4

;hours
		swapf	v_rtc_hrs,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_h1

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_h2

;minutes
		swapf	v_rtc_min,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_n1

		movf	v_rtc_min,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_n2

;seconds
		swapf	v_rtc_sec,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_s1

		movf	v_rtc_sec,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_s2
;
;day in the week
		movf	v_rtc_wday,0
		andlw	B'00000111'
		movwf	v_rtc_wday

		movf	v_rtc_wday,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	rtc_set_Mo

		movf	v_rtc_wday,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	rtc_set_Tu

		movf	v_rtc_wday,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	rtc_set_We

		movf	v_rtc_wday,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	rtc_set_Th

		movf	v_rtc_wday,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	rtc_set_Fr

		movf	v_rtc_wday,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	rtc_set_Sa

		movf	v_rtc_wday,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	rtc_set_Su
;
		return

rtc_set_Mo	movlw	'M'
		movwf	v_rtc_t1
		movlw	'o'
		movwf	v_rtc_t2
		return

rtc_set_Tu	movlw	'T'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

rtc_set_We	movlw	'W'
		movwf	v_rtc_t1
		movlw	'e'
		movwf	v_rtc_t2
		return

rtc_set_Th	movlw	'T'
		movwf	v_rtc_t1
		movlw	'h'
		movwf	v_rtc_t2
		return

rtc_set_Fr	movlw	'F'
		movwf	v_rtc_t1
		movlw	'r'
		movwf	v_rtc_t2
		return

rtc_set_Sa	movlw	'S'
		movwf	v_rtc_t1
		movlw	'a'
		movwf	v_rtc_t2
		return

rtc_set_Su	movlw	'S'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

;-----------------------------------------------------------------------------
rtc_backtrs	nop
;date - day
		movlw	'0'
		subwf	v_rtc_d1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_d2,0
		addwf	v_tmp1,0

		movwf	v_rtc_day
;----------------------------------
;date - month
		movlw	'0'
		subwf	v_rtc_m1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_m2,0
		addwf	v_tmp1,0

		movwf	v_rtc_mnt
;----------------------------------
;date - year
		movlw	'0'
		subwf	v_rtc_y3,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_y4,0
		addwf	v_tmp1,0

		movwf	v_rtc_year
;----------------------------------
;date - day in the week - always use 1; and we will see...
		movlw	'0'
		subwf	v_rtc_wd,0
		movwf	v_rtc_wday

;----------------------------------
;time - hours
		movlw	'0'
		subwf	v_rtc_h1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_h2,0
		addwf	v_tmp1,0

		movwf	v_rtc_hrs
;----------------------------------
;time - minutes
		movlw	'0'
		subwf	v_rtc_n1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_n2,0
		addwf	v_tmp1,0

		movwf	v_rtc_min
;----------------------------------
;time - seconds
		movlw	'0'
		subwf	v_rtc_s1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_s2,0
		addwf	v_tmp1,0

		movwf	v_rtc_sec

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Command Editor
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
spi_editor_st	call	spi_reinit2
spi_editor	call	spi_cs0_dis		;disable CS0

		call	disp_clr2
		call	disp_clr1

		movlw	B'10000000'
		call	send_comm
		call	wm_C
		call	wm_S

ddq002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'45'			;0 = CS0
		btfsc	STATUS,Z
		goto	spi_edit_cs0

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	ddq002

		goto	lg_main_menu
;------------------------------------------------------------------------------
spi_edit_cs0	call	wm_0
		call	wm_sip1
		call	spi_cs0_en		;activate -CS0

		call	d2
		call	d2
		call	d2

		goto	spi_edit2
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
spi_edit2	movlw	D'003'
		movwf	i2ce_cursor

spi_edit3	movf	i2ce_cursor,0
		addlw	B'11000000'
		call	send_comm
		call	wm_dvojbodka

		movf	i2ce_cursor,0
		addlw	D'128'
		call	send_comm
		call	wm_sip1

		clrf	v_b_in
		movf	i2ce_cursor,0
		addlw	B'10000001'
		call	hex_input
;----------------------------------------------------------------------------
		movf	v_comm,0
		xorlw	H'01'			;ESC - back to start menu
		btfsc	STATUS,Z
		goto	spi_editor
;----------------------------------------------------------------------------
		movf	v_b_in,0
		call	spi_send
		call	d2

		movf	i2ce_cursor,0
		addlw	B'11000001'
		call	send_comm
		movf	v_spi_buffi,0
		movwf	kbd_buffer
		call	write_hex

		call	spi_mcc3

		goto	spi_edit3

;----------------------------------------------------------------------------
spi_mcc3	movlw	D'003'
		addwf	i2ce_cursor,1

;we want to have at least 3 characters left
		movf	i2ce_cursor,0
		movwf	data_tmp
		movlw	D'009'			;5 needs to be left, but we need 6 more (to be able to compare bit 5...
		addwf	data_tmp,1

		btfss	data_tmp,D'005'		;we passed the column 24...
		return

		call	d2
		call	d2
		movlw	D'003'
		movwf	i2ce_cursor
		call	disp_clr1_p4
		call	disp_clr2
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Main Menu
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m_spi		call	disp_clr2
m_spi2		call	disp_clr1
;;		call	w_spi

yy002		clrf	kbd_buffer
		call	get_datafk


		movf	kbd_buffer,0
		xorlw	H'1B'			;S = SPI Command Editor
		btfsc	STATUS,Z
		goto	spi_editor_st

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	yy002

		goto	m_spi
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_spi_par	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm
		movlw	high t_spi_par1
		movwf	TBLPTRH
		movlw	low t_spi_par1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	'0'
		btfsc	v_spi_conf,D'004'
		movlw	'1'
		call	send_data
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      SPI Parameters Setup
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
s_spi_par	call	disp_clr1
		call	disp_clr2
s_spi_par2	call	w_spi_par

spp002		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'43'			;I = Idle Clock State toggle
		btfsc	STATUS,Z
		goto	s_spi_icst

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	spp002
		goto	s_spi_par

;------------------------------------------------------------------------------
s_spi_icst	btg	v_spi_conf,D'004'
		call	d2
		call	d2
		goto	s_spi_par2


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; "Debugging Screen for 1-wire ROM Search functionality
;
; first  line shows the last found ROM ID
; second line shows the conflict points
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_execute	clrf	v_ow_dc

		clrf	v_owc0
		clrf	v_owc1
		clrf	v_owc2
		clrf	v_owc3
		clrf	v_owc4
		clrf	v_owc5
		clrf	v_owc6
		clrf	v_owc7

		clrf	v_owd0
		clrf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7

		clrf	v_owdc

		lfsr	FSR0, H'100'
;----------------------------------------------------
ow_rs_next	call	ow_rst
		movlw	H'F0'			;Search ROM
		movwf	ow_buffer
		call	ow_write

		movf	v_owd0,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_0
		movf	v_owcc,0
		movwf	v_owc0


		movf	v_owd1,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_1
		movf	v_owcc,0
		movwf	v_owc1


		movf	v_owd2,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_2
		movf	v_owcc,0
		movwf	v_owc2

		movf	v_owd3,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_3
		movf	v_owcc,0
		movwf	v_owc3

		movf	v_owd4,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_4
		movf	v_owcc,0
		movwf	v_owc4

		movf	v_owd5,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_5
		movf	v_owcc,0
		movwf	v_owc5


		movf	v_owd6,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_6
		movf	v_owcc,0
		movwf	v_owc6


		movf	v_owd7,0
		movwf	v_owdc
		call	ow_rs_byte
		movwf	ow_ROM_7
		movf	v_owcc,0
		movwf	v_owc7

;------------------------------------------------------------
		movf	v_ow_dc,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		return				;no device
;-----------------------------------------------------------

		call	rom_s_store

;-----------------------------------------------------------
;now find last conflict and change the corresponding direction

		movf	v_owd7,0
		movwf	v_owdc
		movf	v_owc7,0
		btfss	STATUS,Z
		goto	ow_rs_next7
		movf	v_owd6,0
		movwf	v_owdc
		movf	v_owc6,0
		btfss	STATUS,Z
		goto	ow_rs_next6
		movf	v_owd5,0
		movwf	v_owdc
		movf	v_owc5,0
		btfss	STATUS,Z
		goto	ow_rs_next5
		movf	v_owd4,0
		movwf	v_owdc
		movf	v_owc4,0
		btfss	STATUS,Z
		goto	ow_rs_next4
		movf	v_owd3,0
		movwf	v_owdc
		movf	v_owc3,0
		btfss	STATUS,Z
		goto	ow_rs_next3
		movf	v_owd2,0
		movwf	v_owdc
		movf	v_owc2,0
		btfss	STATUS,Z
		goto	ow_rs_next2
		movf	v_owd1,0
		movwf	v_owdc
		movf	v_owc1,0
		btfss	STATUS,Z
		goto	ow_rs_next1
		movf	v_owd0,0
		movwf	v_owdc
		movf	v_owc0,0
		btfss	STATUS,Z
		goto	ow_rs_next0

;search finished
		call	ow_rst

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_next7	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next6	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next5	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next4	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next3	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next2	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next1	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------
ow_rs_next0	movwf	v_owcc
		call	ow_rs_nextc
		movwf	v_owd0
		clrf	v_owd1
		clrf	v_owd2
		clrf	v_owd3
		clrf	v_owd4
		clrf	v_owd5
		clrf	v_owd6
		clrf	v_owd7
		goto	ow_rs_next
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_nextc	btfsc	v_owcc,D'007'
		goto	ow_rs_nextcb7
		btfsc	v_owcc,D'006'
		goto	ow_rs_nextcb6
		btfsc	v_owcc,D'005'
		goto	ow_rs_nextcb5
		btfsc	v_owcc,D'004'
		goto	ow_rs_nextcb4
		btfsc	v_owcc,D'003'
		goto	ow_rs_nextcb3
		btfsc	v_owcc,D'002'
		goto	ow_rs_nextcb2
		btfsc	v_owcc,D'001'
		goto	ow_rs_nextcb1
		goto	ow_rs_nextcb0
;------------------------------------------
ow_rs_nextcb7	bsf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb6	bsf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb5	bsf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb4	bsf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb3	bsf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb2	bsf	v_owdc,D'002'
		bcf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb1	bsf	v_owdc,D'001'
		bcf	v_owdc,D'002'
		bcf	v_owdc,D'003'
		bcf	v_owdc,D'004'
		bcf	v_owdc,D'005'
		bcf	v_owdc,D'006'
		bcf	v_owdc,D'007'
		movf	v_owdc,0
		return
;------------------------------------------
ow_rs_nextcb0	clrf	v_owdc
		bsf	v_owdc,D'000'
		movf	v_owdc,0
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
rom_s_store	incf	v_ow_dc,1		;increment "devices found"

		movf	v_ow_dc,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	ow_too_much

		call	w_ows_prog_i

;store the device ROM ID

		movf	ow_ROM_0,0
		movwf	v_owd0
		movwf	POSTINC0

		movf	ow_ROM_1,0
		movwf	v_owd1
		movwf	POSTINC0

		movf	ow_ROM_2,0
		movwf	v_owd2
		movwf	POSTINC0

		movf	ow_ROM_3,0
		movwf	v_owd3
		movwf	POSTINC0

		movf	ow_ROM_4,0
		movwf	v_owd4
		movwf	POSTINC0

		movf	ow_ROM_5,0
		movwf	v_owd5
		movwf	POSTINC0

		movf	ow_ROM_6,0
		movwf	v_owd6
		movwf	POSTINC0

		movf	ow_ROM_7,0
		movwf	v_owd7
		movwf	POSTINC0

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; ROM Search Progress Indicator
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ows_prog_i	movlw	D'128'
		call	send_comm

		call	wm_D
		call	wm_e
		call	wm_v
		call	wm_i
		call	wm_c
		call	wm_e
		call	wm_s
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_u
		call	wm_n
		call	wm_d
		call	wm_dvojbodka
		call	wm_space

		movf	v_ow_dc,0
		movwf	kbd_buffer
		call	write_dec

		call	wm_space
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_rs_byte	nop
		clrf	v_rom_id_c
		clrf	v_owcc

;bit 0 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'000'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfss	ds2484_st,D'005'
		goto	x567_2
		btfsc	ds2484_st,D'006'
		goto	ow_no_device

x567_2		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'000'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'000'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'000'

		btfsc	v_owdc,D'000'
		bcf	v_owcc,D'000'

;bit 1 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'001'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'001'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'001'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'001'

		btfsc	v_owdc,D'001'
		bcf	v_owcc,D'001'
;bit 2 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'002'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'002'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'002'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'002'

		btfsc	v_owdc,D'002'
		bcf	v_owcc,D'002'
;bit 3 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'003'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'003'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'003'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'003'

		btfsc	v_owdc,D'003'
		bcf	v_owcc,D'003'
;bit 4 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'004'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'004'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'004'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'004'

		btfsc	v_owdc,D'004'
		bcf	v_owcc,D'004'
;bit 5 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'005'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'005'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'005'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'005'

		btfsc	v_owdc,D'005'
		bcf	v_owcc,D'005'
;bit 6 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'006'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'006'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'006'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'006'

		btfsc	v_owdc,D'006'
		bcf	v_owcc,D'006'
;bit 7 ---------------------------------
		movlw	H'FF'
		btfss	v_owdc,D'007'
		movlw	H'00'
		movwf	v_ow_act_dir

		call	ow_triplet
		btfsc	ds2484_st,D'007'
		bsf	v_rom_id_c,D'007'

		btfsc	ds2484_st,D'006'
		btg	v_owcc,D'007'
		btfss	ds2484_st,D'005'
		btg	v_owcc,D'007'

		btfsc	v_owdc,D'007'
		bcf	v_owcc,D'007'
;----------------------------------------------------
x999_f		movf	v_rom_id_c,0
		return
;----------------------------------------------------
ow_no_device	movlw	H'FF'
		movwf	v_ow_dc
		return
;----------------------------------------------------
ow_too_much	call	disp_clr1
		call	disp_clr2

;Row 1
		movlw	D'128'
		call	send_comm
		movlw	high t_owtm1
		movwf	TBLPTRH
		movlw	low t_owtm1
		movwf	TBLPTRL
		call	w_send_txt

;Row 2
		movlw	D'192'
		call	send_comm
		movlw	high t_owtm2
		movwf	TBLPTRH
		movlw	low t_owtm2
		movwf	TBLPTRL
		call	w_send_txt


endegelende	goto	endegelende

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;X                       1-Wire ROM Search                                     X
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;subroutines description:
;
;	ow_rs_execute - scan the 1-wire bus and store ROM IDs into
;                       memory bank 2
;                       each ROM ID needs 8 bytes of memory
;                     - Process:
;                          0. the initialization is done:
;                             all direction bits are set to 0
;                          1. 1-wire reset is executed
;                          2. 1-wire ROM Search command is executed
;                          3. 8 x ow_rs_byte is executed starting with LSB
;			   4. first (on the MSb) conflict is found and the
;                             corresponding direction bit is changed from 0 to 1
;                               - if there is no conflict anymore -- search is finished
;                          5. the cycle is executed again from point 1
;
;
;	ow_rs_byte    - execute the scan for one byte of the ROM ID
;                     - for one cycle this soubroutine is executed 8 times
;                     - internaly the routine ow_triplet is used for every bit
;                     - means, the ow_triplet is called 8 times from this routine
;                     - variable v_owdc defines the direction
;                           - is filled with the corresponding v_owd0 - v_owd7
;                             prior to call
;                     - variable v_owcc contains after the execution "list" of
;                       conflicts - for every bit where at least two devices
;                       response with different bit, the corresponding bit
;                       will be set to 1 - but only if the corresponding
;                       direction bit is 0
;       rom_s_store   - sotred the ROM ID to memory bank 2 
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_ow_DS18B20	call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_B
		call	wm_2
		call	wm_0
		return
;-------------------------------------------------------------------------------
w_ow_DS18S20	call	wm_D
		call	wm_S
		call	wm_1
		call	wm_8
		call	wm_S
		call	wm_2
		call	wm_0
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2484_st_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'F0'			;address of the Status Register
		call	i2c_send
;
ds2484_st_read2	nop
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_st
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
ds2484_da_read	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'E1'			;address of the Read Data Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_data
		call	i2c_not_ack
		call	i2c_stop

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_m_start	btfss	v_md_status,D'007'
		return

		call	request_store

		call	ow_rst
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		call	ow_write_spu

		bsf	v_md_status,D'006'
		bcf	v_md_status,D'007'

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
sensorA_read	btfss	v_sensors,D'000'
		return

		movlw	H'00'
		movwf	ee_add
		call	load_ROMID_ee

		movlw	H'00'
		movwf	ee_add
		call	ow_match_rom
		call	ds1820_read_sp

		call	convert_temp
		movwf	v_i1
		return
;-------------------------------------------------
convert_temp	movf	ow_m8,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	s_no_value

;------------------------------------------------
;it is a BS18B20 or MAX31820?
		movf	ow_ROM_0,0
		xorlw	H'28'
		btfss	STATUS,Z
		goto	ct_c1
;pre-process for BS18B20

		rrncf	ow_m1,1
		rrncf	ow_m1,1
		rrncf	ow_m1,1
		bcf	ow_m1,D'007'
		bcf	ow_m1,D'006'
		bcf	ow_m1,D'005'

		movf	ow_m2,0
		movwf	c_tmp
		rrncf	c_tmp,1
		rrncf	c_tmp,1
		rrncf	c_tmp,1
		bcf	c_tmp,D'000'
		bcf	c_tmp,D'001'
		bcf	c_tmp,D'002'
		bcf	c_tmp,D'003'
		bcf	c_tmp,D'004'

		movf	c_tmp,0
		addwf	ow_m1,1
;------------------------------------------------

ct_c1		btfsc	ow_m2,D'007'
		goto	negative_value

		movf	ow_m1,0
		movwf	c_tmp
		rrncf	c_tmp,1
		bcf	c_tmp,D'007'
		movf	c_tmp,0
		return
;-------------------------------------------------
negative_value	nop

		movf	ow_m1,0
		movwf	c_tmp
		comf	c_tmp,1
		rrncf	c_tmp,1

		bsf	c_tmp,D'007'		;sign
		movf	c_tmp,0

		return
;-------------------------------------------------
s_no_value	movlw	H'FF'
		movwf	c_tmp
		return
;-------------------------------------------------------------------------------
sensorB_read	btfss	v_sensors,D'001'
		return

		movlw	H'08'
		movwf	ee_add
		call	load_ROMID_ee

		movlw	H'08'
		movwf	ee_add
		call	ow_match_rom
		call	ds1820_read_sp

		call	convert_temp
		movwf	v_i2

		return
;-------------------------------------------------------------------------------
sensorC_read	btfss	v_sensors,D'002'
		return

		movlw	H'10'
		movwf	ee_add
		call	load_ROMID_ee

		movlw	H'10'
		movwf	ee_add
		call	ow_match_rom
		call	ds1820_read_sp

		call	convert_temp
		movwf	v_i3
		return

;-------------------------------------------------------------------------------
sensorD_read	btfss	v_sensors,D'003'
		return

		movlw	H'18'
		movwf	ee_add
		call	load_ROMID_ee

		movlw	H'18'
		movwf	ee_add
		call	ow_match_rom
		call	ds1820_read_sp

		call	convert_temp
		movwf	v_i4
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ow_match_rom	call	ow_rst

		movlw	H'55'		;Match ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'08'
		movwf	c_tmp

owmr_l1		call	ee_read_byte
		movf	ee_buffer,0
		movwf	ow_buffer
		call	ow_write
		incf	ee_add,1
		decfsz	c_tmp,1
		goto	owmr_l1

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds1820_read_sp	movlw	H'BE'		;Read Scratchpad
		movwf	ow_buffer
		call	ow_write

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m1

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m2

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m3

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m4

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m5

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m6

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m7

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m8

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m9

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;###############################################################################
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;         1-wire communication routines utilizing bridge device DS2484
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;###############################################################################
;-------------------------------------------------------------------------------
;1-wire Reset
;-------------------------------------------------------------------------------
ow_rst		nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'B4'			;command: 1-wire Reset
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop

		bcf	v_md_status,D'006'
		bcf	v_md_status,D'007'

		return
;-------------------------------------------------------------------------------
;1-wire write byte and use strong pull-up (SPU)
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write_spu	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'D2'			;command: Write Device Configuration
		call	i2c_send
		movlw	B'10110100'		;activate SPU bit of cinfiguration register
		call	i2c_send
		call	d55			;1-wire timing
		call	i2c_stop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire write byte
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire read byte
;                   byte will be stored to: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_read		nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'96'			;command: 1-wire Read Byte
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	i2c_stop
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	ds2484_da_read

		movf	ds2484_data,0
		movwf	ow_buffer
		return
;-------------------------------------------------------------------------------
;1-wire Triplet
;                   used for ROM Search algorithm
;
; content of the Status register is stored in: ds2484_st
;     bit 5 (SBR) is the address bit of the device
;     bit 6 (TSB) is the complement of the device address bit
;     bit 7 (DIR) is the "direction taken bit"
;-------------------------------------------------------------------------------
ow_triplet	nop
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'78'			;command: 1-wire Triplet
		call	i2c_send
		movf	v_ow_act_dir,0		;direction
		call	i2c_send
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	d55			;1-wire timing
		call	ds2484_st_read2
		return
;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################
;-------------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          DS1820  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ds1820i2	movlw	B'11000000'
		call	send_comm

		movf	ow_ROM_7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_hviezdicka

		movf	ow_ROM_6,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_5,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_4,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_3,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_2,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		movf	ow_ROM_1,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_minus
		call	wm_minus
		movf	ow_ROM_0,0
		movwf	kbd_buffer
		call	write_hex

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################
;-------------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;                          RTC set
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
rtc_set		nop
		call	disp_clr2
		call	w_rtcm_s
;

		movlw	B'10000000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d1

		movlw	B'10000001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_d2

		movlw	B'10000011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m1

		movlw	B'10000100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_m2

		movlw	B'10001000'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y3

		movlw	B'10001001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_y4

		movlw	B'10001011'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h1

		movlw	B'10001100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_h2

		movlw	B'10001110'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n1

		movlw	B'10001111'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_n2

		movlw	B'10010001'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s1

		movlw	B'10010010'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_s2

		movlw	B'10010100'
		movwf	tmp_ot
		call	sa_key
		movf	b_ax,0
		movwf	v_rtc_wd

		goto	rtc_set		;start cyclic - from the first number again...

rtc_set_set	call	rtc_backtrs	;translate the info on the display to RTC device format
		call	rtc_i2c_write	;write data to RTC device
		goto	lg_system_menu

rtc_set_can	nop
		goto	lg_system_menu		;ESC - no changes to be made

;-----------------------------------------------------------------
;-----------------------------------------------------------------
	
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;---------------------------------------------------------------------------------------------------
;Temperature Logger - Main screen
;---------------------------------------------------------------------------------------------------
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
w_ms001		nop

;Row1: Time & Status

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		btfss	v_md_status2,D'000'
		call	w_rtcm_dt
		btfsc	v_md_status2,D'000'
		call	w_mem_full

		movlw	B'10010100'		;row 1 / col: 20
		call	send_comm
		call	w_m_status

;Row2: Measured temperature

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		btfsc	v_md_status,D'001'
		incf	v_dic,1


		btfss	v_dic,D'004'
		call	w_mtemp
		btfsc	v_dic,D'004'
		call	w_progress


;Hidding the cursor

		movlw	B'10010111'		;row 1 / col: 23
		call	send_comm
		movlw	H'FF'
		btfss	v_md_status,D'007'
		movlw	' '			;mesurement in progress
		call	send_data
		return
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_mem_full	movlw	high t_memf1
		movwf	TBLPTRH
		movlw	low t_memf1
		movwf	TBLPTRL
		call	w_send_txt

		call	wm_space
		call	w_rtcm_dt_t

		clrf	v_dic
		return
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_progress	call	wm_I
		call	wm_D
		call	wm_dvojbodka
		movf	v_file,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_space

		call	wm_R
		call	wm_dvojbodka
		movf	v_rec_msb,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_rec_lsb,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		call	wm_A
		call	wm_d
		call	wm_d
		call	wm_dvojbodka
		movf	v_m_add2,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add1,0
		movwf	kbd_buffer
		call	write_hex
		movf	v_m_add0,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		return
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_mtemp		call	wm_A
		call	wm_dvojbodka
		btfss	v_sensors,D'000'
		call	wm_trippleX
		btfsc	v_sensors,D'000'
		call	wm_tempA
		call	wm_space

		call	wm_B
		call	wm_dvojbodka
		btfss	v_sensors,D'001'
		call	wm_trippleX
		btfsc	v_sensors,D'001'
		call	wm_tempB
		call	wm_space

		call	wm_C
		call	wm_dvojbodka
		btfss	v_sensors,D'002'
		call	wm_trippleX
		btfsc	v_sensors,D'002'
		call	wm_tempC
		call	wm_space

		call	wm_D
		call	wm_dvojbodka
		btfss	v_sensors,D'003'
		call	wm_trippleX
		btfsc	v_sensors,D'003'
		call	wm_tempD
		call	wm_space


		return
;------------------------------------------------
wm_trippleX	call	wm_x
		call	wm_x
		call	wm_x
		return
;------------------------------------------------
wm_tripplem	btfsc	v_usb_sw,D'000'
		goto	wm_usb_tm
		call	wm_minus
		call	wm_minus
		call	wm_minus
		return

wm_usb_tm	movlw	'-'
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		movlw	'-'
		call	i2c2usb
		return

;------------------------------------------------
wm_tempA	movf	v_i1,0
		goto	wm_tempx
;------------------------------------------------
wm_tempB	movf	v_i2,0
		goto	wm_tempx
;------------------------------------------------
wm_tempC	movf	v_i3,0
		goto	wm_tempx
;------------------------------------------------
wm_tempD	movf	v_i4,0
		goto	wm_tempx
;------------------------------------------------
wm_tempx	movwf	kbd_buffer

		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	wm_tripplem
;display the temperature

		movlw	'+'
		btfsc	kbd_buffer,D'007'
		movlw	'-'

		btfss	v_usb_sw,D'000'
		call	send_data
		btfsc	v_usb_sw,D'000'
		call	i2c2usb


		bcf	kbd_buffer,D'007'
		call	write_dec2

		return


;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_m_status	movlw	'K'
		btfss	v_md_status,D'000'
		movlw	'X'			;no keyboard
		call	send_data

		movlw	'M'
		btfss	v_md_status,D'001'
		movlw	'-'			;no measurement in progress
		call	send_data

		movlw	'-'
		btfsc	v_md_status,D'002'
		movlw	'N'
		btfsc	v_md_status,D'003'
		movlw	'E'
		btfsc	v_md_status,D'004'
		movlw	'F'
		call	send_data

		return
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;RTC display date/time in first row
;
w_rtcm_dt2	movlw	B'10000000'
		call	send_comm

w_rtcm_dt	movf	v_rtc_d1,0
		call	send_data
		movf	v_rtc_d2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_m1,0
		call	send_data
		movf	v_rtc_m2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_y1,0
		call	send_data
		movf	v_rtc_y2,0
		call	send_data
		movf	v_rtc_y3,0
		call	send_data
		movf	v_rtc_y4,0
		call	send_data

		call	wm_sls
		
w_rtcm_dt_t	movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_s1,0
		call	send_data
		movf	v_rtc_s2,0
		call	send_data

		return

;-----------------------------------------------------------------
w_rtcm_dt_usb	movf	v_rtc_d1,0
		call	i2c2usb
		movf	v_rtc_d2,0
		call	i2c2usb
		movlw	'.'
		call	i2c2usb
		movf	v_rtc_m1,0
		call	i2c2usb
		movf	v_rtc_m2,0
		call	i2c2usb
		movlw	'.'
		call	i2c2usb
		movf	v_rtc_y1,0
		call	i2c2usb
		movf	v_rtc_y2,0
		call	i2c2usb
		movf	v_rtc_y3,0
		call	i2c2usb
		movf	v_rtc_y4,0
		call	i2c2usb

		movlw	'/'
		call	i2c2usb
		
		movf	v_rtc_h1,0
		call	i2c2usb
		movf	v_rtc_h2,0
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		movf	v_rtc_n1,0
		call	i2c2usb
		movf	v_rtc_n2,0
		call	i2c2usb
		movlw	':'
		call	i2c2usb
		movf	v_rtc_s1,0
		call	i2c2usb
		movf	v_rtc_s2,0
		call	i2c2usb

		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_rtcm		call	w_rtcm_dt2
;
;
		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_S
		call	wm_zat2
		call	wm_e
		call	wm_t
		call	wm_sls
		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_space
		call	wm_space
		call	w_i2c_add

		return
;-----------------------------------------------------------------
w_rtcm_s	call	w_rtcm_dt2

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_S
		call	wm_zat1
		call	wm_e
		call	wm_zat2
		call	wm_t
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C

		return
;-----------------------------------------------------------------
w_rtc_e		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_R
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_s
		call	wm_e
		call	wm_t
		call	wm_space
		call	wm_e
		call	wm_x
		call	wm_e
		call	wm_c
		call	wm_u
		call	wm_t
		call	wm_e
		call	wm_d
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	B'11000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_E
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_R
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------



;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read current date/time
;
rtc_i2c_read	nop

;Step 1 - Start condition
		call	i2c_start

;Step 2: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send

;Step 4: send STOP condition (stop the write)
		call	i2c_stop

;Step 5: START condition
		call	i2c_start

;Step 6: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

;-------------------------------------
;Step 7: read data
;7.1 - sec
		call	i2c_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_ack

;7.2 - min
		call	i2c_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_ack

;7.3 - hours
		call	i2c_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_ack

;7.4 - day in a week
		call	i2c_receive		;received byte
		movwf	v_rtc_wday
		call	i2c_ack

;7.5 - day
		call	i2c_receive		;received byte
		movwf	v_rtc_day
		call	i2c_ack

;7.6 - month
		call	i2c_receive		;received byte
		movwf	v_rtc_mnt
		call	i2c_ack

;7.7 - year
		call	i2c_receive		;received byte
		movwf	v_rtc_year
		call	i2c_not_ack

;Step 8: send STOP condition (stop the read)
		call	i2c_stop

		return

;--------------------------------------------------------------------------------------
;write example date / time to RTC: date = 15.11.2013 time = 14:32:45; day = 5 (Friday)
;--------------------------------------------------------------------------------------
rtc_i2c_write_e	nop

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send



;-------------------------------------
;Step 4: write data
;4.1 - sec
		movlw	B'01000101'		;BCD code of 45
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movlw	B'00110010'		;BCD code of 32
		call	i2c_send

;4.3 - hours
		movlw	B'00010100'		;BCD code of 14
		call	i2c_send

;4.4 - day in a week
		movlw	B'00000101'		;BCD code of 05
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movlw	B'00010101'		;BCD code of 15
		call	i2c_send

;4.6 - month
		movlw	B'00010001'		;BCD code of 11
		call	i2c_send

;4.7 - year
		movlw	B'00010011'		;BCD code of 13
		call	i2c_send

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_i2c_write	nop

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send
;-------------------------------------
;Step 4: write data
;4.1 - sec
		movf	v_rtc_sec,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movf	v_rtc_min,0
		call	i2c_send

;4.3 - hours
		movf	v_rtc_hrs,0
		call	i2c_send

;4.4 - day in a week
		movf	v_rtc_wday,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movf	v_rtc_day,0
		call	i2c_send

;4.6 - month
		movf	v_rtc_mnt,0
		call	i2c_send

;4.7 - year
		movf	v_rtc_year,0
		call	i2c_send

		btfsc	v_i2c_add,D'001'
		call	MCP7940_calib		;for MCP7940 - write calibration data in addition

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return

;-----------------------------------------------------------------------------
MCP7940_calib	movlw	H'80'
		call	i2c_send		;control register (just use the power-up value)

		movlw	D'000'
		call	i2c_send		;write 120 (DEC) to calibration register to speed-up the clock

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;initialize SPI sequencial opeation
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_ee_seq_op	call	spi_cs0_en		;activate -CS0
		movlw	B'00000001'		;send instruction WRMR (Write Mode Rgister)
		call	spi_send
		movlw	B'01000000'		;'01xxxxx' = sequencial mode (power-up default)
		call	spi_send
		call	spi_cs0_dis		;deactivate -CS0
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------


;---------------------------------------------
d332		MOVLW   D'001'
		MOVWF   TIMER2
DEL_LOOP331	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP332	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP332
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP331
		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d1_f		MOVLW   D'255'
		MOVWF   TIMER2
del_loop1f	DECFSZ  TIMER2,F
		GOTO    del_loop1f
		return

;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;------------------------------------
drr2		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5
;R/W = RE1 -- N/C
;E   = RE0 -- RA4

;D0 - D3 -- not connected

;D4  = RD0 -- RA0
;D5  = RD1 -- RA1
;D6  = RD2 -- RA2
;D7  = RD3 -- RA3


disp_init	nop

		clrf	TBLPTRU

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTA
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTA,D'005'
		bcf	PORTA,D'004'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTA,D'004'
		call	dy_comm
		bcf	PORTA,D'004'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------

set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTA,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTA,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTA,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTA,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTA,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTA,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTA,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTA,D'003'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTA,D'004'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTA,D'005'		;RS -> 1 (data)
		bsf	PORTA,D'004'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTA,D'004'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTA,D'004'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTA,D'005'		;RS -> 0 (command)
		bsf	PORTA,D'004'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTA,D'004'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


disp_clr1_p4	movlw	B'10000100'		;line 1 / char4
		call	send_comm
		goto	disp_clrx
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; sending text string to LCD; end of string is recognized with FEh
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
w_send_txt	tblrd*+
		movf	TABLAT,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		return			;end of string reached

		movf	TABLAT,0
		call	send_data
		goto	w_send_txt

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_release1
		movwf	TBLPTRH
		movlw	low t_release1
		movwf	TBLPTRL

		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		movlw	low t_release2
		movwf	TBLPTRL
		call	w_send_txt

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_kbd_found	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_kbd_p
		movwf	TBLPTRH
		movlw	low t_kbd_p
		movwf	TBLPTRL
		call	w_send_txt

		return

;-----------------------------------------------------------------------
w_kbd_nfound	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_kbd_n
		movwf	TBLPTRH
		movlw	low t_kbd_n
		movwf	TBLPTRL
		call	w_send_txt

		return

;-----------------------------------------------------------------------
;-----------------------------------------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_i2c_add	call	wm_A
		call	wm_dvojbodka
		movf	v_i2c_add,0
		movwf	kbd_buffer

;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec2	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next2	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f2

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn2

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn2

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn2		decf	kbd_buffer,1
		goto	conv_next2
;-------------------------------------------------
tst_conv_f2	movf	v_tst2,0
		btfss	v_usb_sw,D'000'
		call	send_data
		btfsc	v_usb_sw,D'000'
		call	i2c2usb

		movf	v_tst3,0
		btfss	v_usb_sw,D'000'
		call	send_data
		btfsc	v_usb_sw,D'000'
		call	i2c2usb
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
wu_0		movlw	'0'
		call	i2c2usb
		return

wu_1		movlw	'1'
		call	i2c2usb
		return

wu_2		movlw	'2'
		call	i2c2usb
		return

wu_3		movlw	'3'
		call	i2c2usb
		return

wu_4		movlw	'4'
		call	i2c2usb
		return

wu_5		movlw	'5'
		call	i2c2usb
		return

wu_6		movlw	'6'
		call	i2c2usb
		return

wu_7		movlw	'7'
		call	i2c2usb
		return

wu_8		movlw	'8'
		call	i2c2usb
		return

wu_9		movlw	'9'
		call	i2c2usb
		return

wu_A		movlw	'A'
		call	i2c2usb
		return

wu_B		movlw	'B'
		call	i2c2usb
		return

wu_C		movlw	'C'
		call	i2c2usb
		return

wu_D		movlw	'D'
		call	i2c2usb
		return

wu_E		movlw	'E'
		call	i2c2usb
		return

wu_F		movlw	'F'
		call	i2c2usb
		return

;-------------------------------------------------------------------------------
write_hex_usb	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wu_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wu_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wu_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wu_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wu_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wu_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wu_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wu_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wu_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wu_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wu_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wu_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wu_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wu_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wu_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wu_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wu_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wu_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wu_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wu_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wu_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wu_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wu_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wu_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wu_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wu_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wu_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wu_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wu_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wu_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wu_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wu_F

		return
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; send data to the keyboard
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;RB0 = kbd clock
;RB1 = kbd data
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
send_ck		nop
		bsf	v_md_status,D'000'
		clrf	kbd_parity
		bsf	kbd_parity,D'000'

;-----------------------------------------------------------
;1.1. set RB7 (keaboard clock) to output ----------------------
		bcf	TRISB,D'007'
;-----------------------------------------------------------
;1.2. put KBD CLK to 0 ----------------------------------------
		bcf	PORTB,D'007'
		call	d57		;wait a while
;-----------------------------------------------------------
;2. put KBD DATA to 0 ---------------------------------------
		bcf	TRISB,D'006'

		bcf	PORTB,D'006'
;-----------------------------------------------------------
;3. release KBD CLK -----------------------------------------
		bsf	PORTB,D'007'

		bsf	TRISB,D'007'
;-----------------------------------------------------------
;4. wait for keayboard to bring the kbd clk low ------------
		movlw	D'255'
		movwf	v_tmp1

		movlw	B'00110000'		;Frequency = 1MHz  ( OSCON<7:5> = 011 )
		movwf	OSCCON

sck01		decfsz	v_tmp1,1
		goto	sck01_c
		goto	kbd_time_out		;time out - probably no keyboard connected
sck01_c		btfsc	PORTB,D'007'
		goto	sck01


		movlw	B'01100000'		;Frequency = 8MHz  ( OSCON<7:5> = 110 )
		movwf	OSCCON

		movf	kbd_out,0
		andlw	B'00000001'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000010'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00000100'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00001000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00010000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'00100000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'01000000'
		call	send_kbd_bit

		movf	kbd_out,0
		andlw	B'10000000'
		call	send_kbd_bit

;now send the parity bit...
		movf	kbd_parity,0
		andlw	B'00000001'
		call	send_kbd_bit

;		movlw	B'00000001'		;stop bit
;		andlw	B'00000001'
;		call	send_kbd_bit

		bsf	TRISB,D'006'		;data as input

w0079		btfsc	PORTB,D'007'
		goto	w0079

w0080		btfss	PORTB,D'007'
		goto	w0080

w0081		btfsc	PORTB,D'007'
		goto	w0081

w0082		btfss	PORTB,D'007'
		goto	w0082

		return


kbd_time_out	bcf	v_md_status,D'000'

		movlw	B'01100000'		;Frequency = 8MHz  ( OSCON<7:5> = 110 )
		movwf	OSCCON

		return

;-----------------------------------------------------------
send_kbd_bit	btfsc	STATUS,Z
		goto	send_kbd0
		goto	send_kbd1

send_kbd0	bcf	PORTB,D'006'

w0077		btfss	PORTB,D'007'
		goto	w0077

w0078		btfsc	PORTB,D'007'
		goto	w0078

		return

send_kbd1	bsf	PORTB,D'006'
		movlw	B'00000001'
		xorwf	kbd_parity,1
		goto	w0077
;-----------------------------------------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;get a transmission from the keyboard
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

get_datafk	nop
		btfsc	PORTB,D'007'
		goto	kbd_err01		;clock level is 1 instead of expected 0
		btfsc	PORTB,D'006'
		goto	kbd_err02		;data level is 1 instead of expected 0 (start bit)
;
get_datafk2	nop
		clrf	kbd_buffer

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'000'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'001'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'002'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'003'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'004'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'005'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'006'

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		bsf	kbd_buffer,D'007'

;get parity bit

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		nop

;compare parity bit

		nop				;not now.... :-)

;wait for communication end - last bit should be "1"

		call	get_bitfk
		btfsc	kbd_bb,D'006'
		nop

		nop				;no check right now...

		return				;keyboard code is in kbd_buffer

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;get bit from the keyboard during transmission
get_bitfk	nop
kbd_l2		btfss	PORTB,D'007'
		goto	kbd_l2			;wait to 1 on clock line
kbd_l3		btfsc	PORTB,D'007'
		goto	kbd_l3			;and now wait to 0 on clock
;
		movf	PORTB,0
		andlw	B'01000000'		;now we have the bit in W1
		movwf	kbd_bb

		return
;-----------------------------------------------------------------------------
kbd_err01	nop
		return
;-----------------------------
kbd_err02	nop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;in W is the command to be send to display (posintion in display)
;
;output will be shifted to the variable		'v_b_in'
;value of the variable				'v_comm' will be:
;
;							ESC		01
;							ENTER		02
;							F10		03
;							other		FF
;------------------------------------------------------------------------------
hex_input	movwf	v_cursor
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'11110000'
		andwf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1
		rrcf	b_ax_h,1

		call	sa_key_a
		movf	v_comm,0
		btfss	STATUS,Z
;exit - something happens
		goto	hi_exit_h

		movf	b_ax,0
		call	send_data
		call	hi_trans_h

		incf	v_cursor,0
		movwf	tmp_ot

		movf	v_b_in,0
		movwf	b_ax_h
		movlw	B'00001111'
		andwf	b_ax_h,1

		call	sa_key_a
		movf	v_comm,0
		btfss	STATUS,Z
		goto	hi_exit_l

		movf	b_ax,0
		call	send_data
		call	hi_trans_l
		return

;now we have to translate the value back to HEX

hi_exit_h	call	hi_trans_h
		return

hi_exit_l	call	hi_trans_l
		return


hi_trans_h	rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		rlcf	b_ax_h,1
		movlw	B'11110000'
		andwf	b_ax_h,1
		movlw	B'00001111'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

hi_trans_l	movlw	B'00001111'
		andwf	b_ax_h,1
		movlw	B'11110000'
		andwf	v_b_in,1
		movf	b_ax_h,0
		addwf	v_b_in,1
		return

;-------------------------------------------------
hi_leave_esc	movlw	H'01'
		movwf	v_comm
		call	dr77
		return

hi_leave_enter	movlw	H'02'
		movwf	v_comm
		call	dr77
		return

hi_leave_F10	movlw	H'03'
		movwf	v_comm
		call	dr77
		return

hi_leave_oth	movlw	H'FE'
		movwf	v_comm
		call	dr77
		return
;-------------------------------------------------

;--------------------------------------------------------------------------
sa_key_a	movlw	H'00'
		movwf	v_comm

		movf	tmp_ot,0
		call	send_comm

sa_key3_a	btfss	PORTB,D'007'
		goto	sa_key2_a
		goto	sa_key3_a

;
sa_key2_a	clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'		;ESC
		btfsc	STATUS,Z
		goto	hi_leave_esc

		movf	kbd_buffer,0
		xorlw	H'5A'		;ENTER
		btfsc	STATUS,Z
		goto	hi_leave_enter

		movf	kbd_buffer,0
		xorlw	H'09'		;F10
		btfsc	STATUS,Z
		goto	hi_leave_F10

		movf	kbd_buffer,0
		xorlw	H'74'		;->
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'6B'		;<-
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'72'		;arrow down
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'75'		;arrow up
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'2D'		;R = read EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth

		movf	kbd_buffer,0
		xorlw	H'1D'		;W = read/write EEPROM from 00-00
		btfsc	STATUS,Z
		goto	hi_leave_oth



		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		movf	kbd_buffer,0
		xorlw	H'1C'		;A
		btfsc	STATUS,Z
		goto	set_avA

		movf	kbd_buffer,0
		xorlw	H'32'		;B
		btfsc	STATUS,Z
		goto	set_avB

		movf	kbd_buffer,0
		xorlw	H'21'		;C
		btfsc	STATUS,Z
		goto	set_avC

		movf	kbd_buffer,0
		xorlw	H'23'		;D
		btfsc	STATUS,Z
		goto	set_avD

		movf	kbd_buffer,0
		xorlw	H'24'		;E
		btfsc	STATUS,Z
		goto	set_avE

		movf	kbd_buffer,0
		xorlw	H'2B'		;F
		btfsc	STATUS,Z
		goto	set_avF

		goto	sa_key3_a
;-----------------------------------
set_av0		movlw	'0'
		movwf	b_ax
		movlw	H'00'
		movwf	b_ax_h
		call	dr77
		return

set_av1		movlw	'1'
		movwf	b_ax
		movlw	H'01'
		movwf	b_ax_h
		call	dr77
		return

set_av2		movlw	'2'
		movwf	b_ax
		movlw	H'02'
		movwf	b_ax_h
		call	dr77
		return

set_av3		movlw	'3'
		movwf	b_ax
		movlw	H'03'
		movwf	b_ax_h
		call	dr77
		return

set_av4		movlw	'4'
		movwf	b_ax
		movlw	H'04'
		movwf	b_ax_h
		call	dr77
		return

set_av5		movlw	'5'
		movwf	b_ax
		movlw	H'05'
		movwf	b_ax_h
		call	dr77
		return

set_av6		movlw	'6'
		movwf	b_ax
		movlw	H'06'
		movwf	b_ax_h
		call	dr77
		return

set_av7		movlw	'7'
		movwf	b_ax
		movlw	H'07'
		movwf	b_ax_h
		call	dr77
		return

set_av8		movlw	'8'
		movwf	b_ax
		movlw	H'08'
		movwf	b_ax_h
		call	dr77
		return

set_av9		movlw	'9'
		movwf	b_ax
		movlw	H'09'
		movwf	b_ax_h
		call	dr77
		return

set_avA		movlw	'A'
		movwf	b_ax
		movlw	H'0A'
		movwf	b_ax_h
		call	dr77
		return

set_avB		movlw	'B'
		movwf	b_ax
		movlw	H'0B'
		movwf	b_ax_h
		call	dr77
		return

set_avC		movlw	'C'
		movwf	b_ax
		movlw	H'0C'
		movwf	b_ax_h
		call	dr77
		return

set_avD		movlw	'D'
		movwf	b_ax
		movlw	H'0D'
		movwf	b_ax_h
		call	dr77
		return

set_avE		movlw	'E'
		movwf	b_ax
		movlw	H'0E'
		movwf	b_ax_h
		call	dr77
		return

set_avF		movlw	'F'
		movwf	b_ax
		movlw	H'0F'
		movwf	b_ax_h
		call	dr77
		return


dr77		goto	d2
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------

;---------------------------------------------------------------------------
w_otaznik	call	send_comm
;
		call	wm_otaznik
;
		movf	tmp_ot,0
		call	send_comm
		return
;---------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
sa_key		nop

		call	w_rtcm_dt2
		movf	tmp_ot,0
		call	w_otaznik

sa_key3		btfss	PORTB,D'007'
		goto	sa_key2
		goto	sa_key3

;
sa_key2		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC = return to menu
		btfsc	STATUS,Z
		goto	rtc_set_can		;"RTC set" cancel

		movf	kbd_buffer,0
		xorlw	H'24'			;E = Set
		btfsc	STATUS,Z
		goto	rtc_set_set		;RTC set the date and time

		movf	kbd_buffer,0
		xorlw	H'45'		;0
		btfsc	STATUS,Z
		goto	set_av0

		movf	kbd_buffer,0
		xorlw	H'16'		;1
		btfsc	STATUS,Z
		goto	set_av1

		movf	kbd_buffer,0
		xorlw	H'1E'		;2
		btfsc	STATUS,Z
		goto	set_av2

		movf	kbd_buffer,0
		xorlw	H'26'		;3
		btfsc	STATUS,Z
		goto	set_av3

		movf	kbd_buffer,0
		xorlw	H'25'		;4
		btfsc	STATUS,Z
		goto	set_av4

		movf	kbd_buffer,0
		xorlw	H'2E'		;5
		btfsc	STATUS,Z
		goto	set_av5

		movf	kbd_buffer,0
		xorlw	H'36'		;6
		btfsc	STATUS,Z
		goto	set_av6

		movf	kbd_buffer,0
		xorlw	H'3D'		;7
		btfsc	STATUS,Z
		goto	set_av7

		movf	kbd_buffer,0
		xorlw	H'3E'		;8
		btfsc	STATUS,Z
		goto	set_av8

		movf	kbd_buffer,0
		xorlw	H'46'		;9
		btfsc	STATUS,Z
		goto	set_av9

		goto	sa_key3

;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------






;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines - new
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

i2c_time_out	goto	i2c_ee

;-----------------------------------------------------------------------------
i2c_start	movlw	D'255'
		movwf	TIMER2

		bcf	PIR1,SSP1IF
		bsf	SSP1CON2,SEN

wait1		dcfsnz	TIMER2,1
		goto	i2c_time_out
		btfss	PIR1,SSP1IF
		goto	wait1
		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bcf	PIR1,SSP1IF
		movwf	SSP1BUF
wait2		btfss	PIR1,SSP1IF
		goto	wait2
		bcf	PIR1,SSP1IF
		btfss	SSP1CON2,ACKSTAT
		goto	ack_received
		goto	ack_not_rec
;------------------------------------------------------------------
ack_received	bsf	PORTA,D'006'		;turn on green LED
		goto	ccc2
;------------------------------------------------------------------
ack_not_rec	bsf	PORTA,D'007'		;turn on red LED
		goto	ccc2
;------------------------------------------------------------------
ccc2		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
xi2c_start	bsf	SSP1CON2,SEN
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	SSP1CON2,PEN
		call	d55
		bcf	PORTA,D'006'
		bcf	PORTA,D'007'
		return
;-----------------------------------------------------------------------------
i2c_ack		bcf	SSP1CON2,ACKDT			;ACK
		bsf	SSP1CON2,ACKEN
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	SSP1CON2,ACKDT			;NOT ACK
		bsf	SSP1CON2,ACKEN
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	SSP1CON2,RCEN
		call	d55
		movf	SSP1BUF,0
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
xi2c_send	movwf	SSP1BUF
		bsf	PORTA,D'006'
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	SSP1CON2,RSEN
		call	d55
		return
;-----------------------------------------------------------------------------


;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;----------------------------------------------------------------------------
dr01s		MOVLW   D'010'		;original value was 010
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;SPI Interface initialization
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;reset SPI for based on settings

spi_reinit1	clrf	SSP2CON1
		movf	v_spi_conf,0		;use the stored value
						;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 1/0    -> Idle state for clock
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;reset SPI for EEPROM
spi_reinit2	clrf	SSP2CON1
		nop
		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
spi_init	nop

; ----------------------------------------------------
; SPI initilization on MSSP2
;
;  RB1 = SCK2
;  RB2 = SDI2
;  RB3 = SDO2
;
; ----------------------------------------------------

		bsf	PORTB,D'004'		;CS = 1

		bsf	TRISB,D'002'		;SDI2 (RB2 -- input)
		bcf	TRISB,D'003'		;SDO2 (RB3 -- output)
		bcf	TRISB,D'001'		;SCK2 (RB1 -- output for master mode)
		bcf	TRISB,D'004'		;CS   (RB4 -- output - CS)

;set up SPI BUS for communication as a master for MSSP2
		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	v_spi_conf
		movwf	SSP2CON1


		clrf	SSP2STAT
						;bit 7   = 0 -> Input data sampled at middle of data output time
						;bit 6   = 0 -> Transmit occurs on transition from Idle to active clock state

		bsf	SSP2STAT,D'006'

		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2

		return

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C Interface initialization
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_init	nop
;set up I2C BUS for communication as a master for MSSP1
		movlw	B'00101000'		;see Datasheet Page 260
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;I2C communication error - time out occured during I2C I/O operations
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
i2c_ee		call	disp_clr1
		call	disp_clr2
		call	w_i2c_ee

ee02		clrf	kbd_buffer
		call	get_datafk

		movf	kbd_buffer,0
		xorlw	H'76'			;ESC - back to start menu
		btfss	STATUS,Z
		goto	ee02

		reset

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
w_i2c_ee	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movlw	high t_i2c_ee1
		movwf	TBLPTRH
		movlw	low t_i2c_ee1
		movwf	TBLPTRL
		call	w_send_txt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_zat2

		return

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; SPI communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_cs0_en	bcf	PORTB,D'004'
		return		
;--------------------------------------
spi_cs0_dis	bsf	PORTB,D'004'
		return		

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_send	bcf	SSP2CON1,SSPOV		;just for sure...
		bcf	SSP2STAT,BF		;just for sure...

		movwf	SSP2BUF
spil2		btfss	SSP2STAT,BF
		goto	spil2
		movf	SSP2BUF,0
		movwf	v_spi_buffi

		return
;-----------------------------------------------------------------------------

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END